<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    wishlist-dot-com-for-woocommerce
 * @subpackage wishlist-dot-com-for-woocommerce/admin
 * @author     WishList.com, Inc. <plugins@wishlist.com>
 */

class Wishlist_Dot_Com_For_Woocommerce_Admin {

    /**
     * Common functionality used by Wishlist_Dot_Com_For_Woocommerce_Public and Wishlist_Dot_Com_For_Woocommerce_Admin
     *
     */
    use wishlistdotcom_woo_trait;

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $options
     */
    private $options;
    private $options_all;

    private $connect_wldcom_section_name;
    private $featured_products_count;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        //the admin tab that's used for connecting this plugin to WishList.com
        $this->connect_wldcom_section_name = 'wishlistdotcom_woo_connect_wldcom';
        $this->featured_products_count = 8;


        if(!isset($this->options_all)){
            $this->options_all = $this->get_options('all');
        }

        // add a post display state for special WC pages.
        add_filter( 'display_post_states', array( $this, 'add_display_post_states' ), 10, 2 );

    }

    /**
     * Add a post display state for wishlistdotcom_woo pages in the list of pages.
     *
     * @param array   $post_states An array of post display states.
     * @param WP_Post $post        The current post object.
     */
    public function add_display_post_states( $post_states, $post ) {
        if ( (int) get_option( WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION ) === $post->ID ) {
            $post_states['wishlistdotcom_woo_hp'] = 'WishList Homepage';
        }
        return $post_states;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        if(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_USE_MIN_CSS){
            $ext = '.min.css';
        }else{
            $ext = '.css';
        }
        //load after exopite-simple-options-framework (styles.css) which is used on our admin page only
        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wishlist-dot-com-for-woocommerce-admin'.$ext, array('exopite-simple-options-framework'), $this->version, 'all' );

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        //load scripts for our admin pages only
        $current_screen = get_current_screen();
        if($current_screen && str_contains($current_screen->id,WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG)){
            if(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_USE_MIN_JS){
                $ext = '.min.js';
            }else{
                $ext = '.js';
            }
            $script_handle = $this->plugin_name;
            wp_enqueue_script( $script_handle, plugin_dir_url( __FILE__ ) . 'js/wishlist-dot-com-for-woocommerce-admin'.$ext, array( 'jquery' ), $this->version, false );

            wp_localize_script(
                $script_handle,
                'wishlistdotcom_object_admin',
                array(
                    'nonce'     => array(
                        'post_options_to_wlcom_rec_ajax' => wp_create_nonce( 'post_options_to_wlcom_rec_ajax' ),
                        'get_product_droppown_rec_ajax' => wp_create_nonce( 'get_product_droppown_rec_ajax' ),
                    ),
                )
            );
        }
    }

    /**
     * Begin options using Exopite-Simple-Options-Framework
     */
    public function create_menu() {

        $valid_api_key = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_valid_api_key');
        if($valid_api_key != 'yes_str' && isset($_GET['section']) && $_GET['section'] != $this->connect_wldcom_section_name){
            //upate tab key
            $cur_url = admin_url() . '?' .$_SERVER['QUERY_STRING'];

            $cur_url = remove_query_arg('section', $cur_url);
            $cur_url = add_query_arg('section', $this->connect_wldcom_section_name, $cur_url);

            //go to correct tab with correct section querysting
            wp_redirect($cur_url);
        }

        $valid_api_key = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_valid_api_key');
        $default_values = $this->get_field_defaults__options('all');
    
        $store_id_a = wishlistdotcom_woo_store_id();
        $api_domain = $store_id_a['api_domain'];
        $wlcom_your_store_url = $store_id_a['store_url'];
        $wlcom_store_shop_url = $store_id_a['store_url_no_scheme'];
        $store_domain = $store_id_a['store_domain'];
        if(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_ENV == 'dev'){
            $wldcom_domain = 'dev.wishlist.com';
        }elseif(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_ENV == 'staging'){
            $wldcom_domain = 'staging.wishlist.com';
        }else{
            $wldcom_domain = 'www.wishlist.com';
        }
        $wlcom_store_listing_preview_url = 'https://'.$wldcom_domain.'/wooCommApi?api_action=store_listing&store_domain='.$store_domain;

        $wl_hp_page_id = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION);
        $wl_hp_url = get_permalink($wl_hp_page_id);

        $wlcom_store_email = get_option('woocommerce_email_from_address');
        if(empty($wlcom_store_email)){
            $wlcom_store_email = get_option('admin_email');
        }
        $button_shortcode__product_page = "[wishlistdotcom_woo_button_product_page]";
        $button_shortcode__product_loop = "[wishlistdotcom_woo_button_product_loop]";
        $button_shortcode__cart = "[wishlistdotcom_woo_button_cart]";
        $custom_css_description = '
            You can also design all elements using custom CSS.
            <br><br>
            <button type="button" class="exopite-sof-btn wishlistdotcom_woo-button-md wishlistdotcom_woo-button-md-js" data-toptab="wishlistdotcom_woo_WishList_custom_css">Edit Custom CSS</button>
        ';
        $button_style_options = [
            'theme_button'    => 'Button with theme style',
            'custom_button'    => 'Button with custom style',
            'custom_button_using_css'    => 'Button using custom css',
            'text_link'    => 'Text link',
        ];
        $button_style_options_default = $button_style_options;

        $button_position_options = [
            'wishlist_button_after_add_to_cart_button'  => "After \"Add to cart\" button",
            'wishlist_button_after_product_info'  => "After product information",
            'wishlist_button_after_thumbnails'  => "After thumbnails",
            'wishlist_button_after_summary'  => "After summary",
            'wishlist_button_use_shortcode'  => "Use shortcode",
            'wishlist_button_use_javascript'  => "Use Javascript",
            'wishlist_button_hide'  => "Hidden",
        ];
        $button_position_options__product_page = [];
        foreach ($button_position_options as $key=>$val){
            $button_position_options__product_page[$key] = $val;
        }
        $button_position_options = [
            'wishlist_button_before_add_to_cart_button'  => "Before \"Add to cart\" button",
            'wishlist_button_after_add_to_cart_button'  => "After \"Add to cart\" button",
            'wishlist_button_use_shortcode'  => "Use shortcode",
            'wishlist_button_use_javascript'  => "Use Javascript",
            'wishlist_button_hide'  => "Hidden",
        ];
        $button_position_options__product_loop = [];
        foreach ($button_position_options as $key=>$val){
            $button_position_options__product_loop[$key] = $val;
        }
        $button_position_options = [
            'wishlist_button_above_right'  => "Above cart, on right",
            'wishlist_button_above_left'  => "Above cart, on left",
            'wishlist_button_above_centered'  => "Above cart, centered",
            'wishlist_button_below_right'  => "Below cart, on right",
            'wishlist_button_below_left'  => "Below cart, on left",
            'wishlist_button_below_centered'  => "Below cart, centered",
            'wishlist_button_use_shortcode'  => "Use shortcode",
            'wishlist_button_use_javascript'  => "Use Javascript",
            'wishlist_button_hide'  => "Hidden",
        ];
        $button_position_options__cart = [];
        foreach ($button_position_options as $key=>$val){
            $button_position_options__cart[$key] = $val;
        }

        //get the add2wishlist button icons
        $svg_icons = $this->wishlistdotcom_woo_get_add2wishlist_icons_drop();
        foreach ($svg_icons as $icon){
            if(!empty($icon['source'])){
                $svg_icons_options[$icon['value']] = ['value'=>$icon['name'],'attribute'=>'data-type="'.esc_attr($icon['type']).'" data-source="'.esc_attr($icon['source']).'"'];
            }else{
                $svg_icons_options[$icon['value']] = ['value'=>$icon['name'], 'attribute'=>'data-type="'.esc_attr($icon['type']).'"'];
            }
        }

        $wlcom_featured_product_skus_options = [];
        foreach ($this->options_all['wlcom_featured_product_skus'] as $item){
            //concactenate sku^text so it's stored together so we don't have to do a query to display later
            $item_a = explode('^',$item);
            $wlcom_featured_product_skus_options[$item] = $item_a[1];
        }

        $font_size_dropdown = $this->get_front_size_dropdown();
        $font_size_dropdown_footer = $this->get_front_size_dropdown('text');

        $store_id_a = wishlistdotcom_woo_store_id();
        $store_url = $store_id_a['store_url'];
        $wlcom_plgn_plugin_display_status = $this->options_all['wlcom_plgn_plugin_display_status'];

        $service_level_key = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_service_level';
        $service_level = get_option($service_level_key);

        $display_status_options = [
            'status_offline'   => "Off. Customers can't see your store's WishList.",
            'status_testing'    => "Testing - Only administrators, and those who can edit pages, can see your store's WishList.",
            'status_live'   => "Live - Customers can see your store's WishList",
        ];

        /*
        * Create a submenu page under Plugins.
        * Framework also add "Settings" to your plugin in plugins list.
        */
        $config_submenu = array(

            'type'              => 'menu', // Required, menu or metabox
            'id'                => $this->plugin_name, // Required, meta box id, unique per page, to save: get_option( id )
            //'parent'            => 'plugins.php', // Required, sub page to your options page
            // 'parent'            => 'edit.php?post_type=your_post_type',
            'menu_title'        =>'WishList.com',
            'submenu'           => false, // Required for submenu
            'title'             => esc_html__( 'WishList.com for WooCommerce', $this->plugin_name ), //The name of this page
            'capability'        => 'manage_options', // The capability needed to view the page
            'plugin_basename'   => plugin_basename( plugin_dir_path( __DIR__ ) . $this->plugin_name . '.php' ),
            'tabbed'            => true,
            'icon'              => plugin_dir_url( __FILE__ ) . 'images/wishlist_bubble_white.svg',
            'search_box'        => false,
            'multilang'         => false,

            //WLDC add our admin link as a settings link that's to the left of the Activate/Deactive link
            'settings_link'     =>admin_url( 'admin.php?page='.WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG ),

        );

        if($valid_api_key != 'yes_str'){
            $section_tab_css = 'wishlistdotcom_woo-tab-grayed-out';
        }else{
            $section_tab_css = '';
        }

        //licensing
        $params = [];
        $params['default_values'] = $default_values;
        $params['store_id_a'] = $store_id_a;
        $params['wlcom_plgn_plugin_display_status'] = $wlcom_plgn_plugin_display_status;
        $params['service_level'] = $service_level;
        $params['valid_api_key'] = $valid_api_key;
        $params['wlcom_store_shop_url'] = $wlcom_store_shop_url;
        $fields[] = $this->get_connect_to_wishlist_field($params);

        $section_admin_fields_for_insert = [];

        /**
         * General settings
         */
        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_general_settings',
            'title'  => 'General Settings',
            'section_title'  => 'General Settings',
            'section_tab_css'  => $section_tab_css,
            'fields' => array(
                array(
                    'type'    => 'content',
                    'content' => "WishList display status",
                    'description' => "This setting controls the visibility of your store's WishList. Use the \"Testing\" mode
                    to test your WishList before you make it live for your customers.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'id'          => 'wlcom_plgn_plugin_display_status',
                    'type'        => 'radio',
                    'title'       => "Set the status of this plugin",
                    'description'    => "\"Testing\" allows you to test the plugin prior to going live. Don't forget to click \"Save Settings\" after making changes.",
                    'options' => $display_status_options,
                    'default'   => $default_values['wlcom_plgn_plugin_display_status'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom',
                ),

                array(
                    'type'    => 'content',
                    'content' => "Your store's listing on WishList.com",
                    'description' => "
                        These settings control your store's listing on WishList.com
                        <p><a class='button button-primary exopite-sof-button' href='".$wlcom_store_listing_preview_url."' target='store_listing_preview'>Preview</a></p>
                        ",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //store name
                array(
                    'id'          => 'wlcom_your_store_name',
                    'type'        => 'text',
                    'title'          => "Your store's name",
                    'before'      => "This can be your domain name, or your business name",
                    'default'   => $default_values['wlcom_your_store_name'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom'
                ),
                array(
                    'id'          => 'wlcom_plgn_your_store_logo',
                    'type'        => 'image',
                    'title'          => "Your store's logo",
                    'description'          => "Used in various places, like your \"Add to WishList\" screens and within WishList.com's shopping section.",
                    'default'   => $default_values['wlcom_plgn_your_store_logo'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom'
                ),
                array(
                    'id'          => 'wlcom_plgn_your_store_icon',
                    'type'        => 'image',
                    'title'          => "Your store's icon",
                    'description'          => "Used in various places, like your \"Add to WishList\" screens and within WishList.com's shopping section. Used for responsive display for mobile devices",
                    'after'    => 'roughly 45px square',
                    'default'   => $default_values['wlcom_plgn_your_store_icon'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom'
                ),

                array(
                    'id'          => 'wlcom_your_store_description',
                    'name'          => 'wlcom_your_store_description',
                    'type'        => 'textarea',
                    'title'       => "Your store's description",
                    'description'          => "Used in various places, like within WishList.com's shopping section.",
                    'attributes'    => array(
                        'rows' => '5',
                    ),
                    'default'          => $default_values['wlcom_your_store_description'],
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area',
                    'wrap_class' => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_your_store_categories',
                    'name'          => 'wlcom_your_store_categories',
                    'type'        => 'textarea',
                    'title'       => "Categories that describe your store",
                    'before'         => 'Comma separated list of category names',
                    'description'          => "Like: Women's Apparel, Home Appliances. Used in various places, like within WishList.com's shopping section.",
                    'attributes'    => array(
                        'rows' => '2',
                    ),
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area',
                    'wrap_class' => 'no-border-bottom',
                ),
                array(
                    'id'          => 'wlcom_your_store_tags',
                    'name'          => 'wlcom_your_store_tags',
                    'type'        => 'textarea',
                    'title'       => "Tags that describe your store",
                    'before'         => 'Comma separated list of tags',
                    'description'          => "Like: Fashion, Trendy, Women's Apparel, Home Tech. Used in various places, like within WishList.com's shopping section.",
                    'attributes'    => array(
                        'rows' => '2',
                    ),
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area',
                    'wrap_class' => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_store_shop_url',
                    'type'        => 'hidden',
                    'name'          => 'wlcom_store_shop_url',
                    'hidden_value'   => $wlcom_store_shop_url,
                ),

                array(
                    'id'          => 'wlcom_store_email',
                    'type'        => 'hidden',
                    'name'          => 'wlcom_store_email',
                    'hidden_value'   => $wlcom_store_email,
                ),

                array(
                    'id'          => 'wlcom_website_is_featured',
                    'type'        => 'radio',
                    'title'       => "Feature your store within WishList.com's shopping section",
                    'description'    => "Your store will be featured on WishList.com for shoppers to find.
                    The featured listing includes priority placement in search results and featured products from below.
                    <br><br>This listing is separate from your store's WishLists, which are always available on WishList.com and your store.
                    ",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'    => "No",
                    ),
                    'default'   => $default_values['wlcom_list_on_wishlistdotcom'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_featured_product_skus',
                    'type'        => 'select',
                    'title'          => "Products to feature with your store's listing on WishList.com",
                    'before'         => "Select ".$this->featured_products_count." products to feature",
                    'attributes' => array(
                        'data-select2-sku' => 'yes',
                        'multiple' => true,
                    ),
                    'options' => $wlcom_featured_product_skus_options,
                    'class'     => 'choice--off select2-selection--multiple wishlistdotcom_woo-form-control',
                    'style'    => 'fancy--off',
                    'wrap_class'   => 'select2-container--bootstrap-5 wishlistdotcom_woo-form-select2 no-border-bottom',
                ),
                array(
                    'id'          => 'wlcom_featured_product_skus__hidden_select2',
                    'type'        => 'hidden',
                    'hidden_value'   => '',
                ),
            ),
        );

        //add commone buttons using get_admin_fields__button
        $params = [];
        $params['section_title'] ="Product page \"Add to WishList\" button";
        $params['option_suffix'] = 'product_page';
        $params['position_title'] = "Position of \"Add to WishList\" on the product page";
        $params['button_position_options'] = $button_position_options__product_page;
        $params['button_shortcode'] = $button_shortcode__product_page;
        $params['button_style_options'] = $button_style_options;
        $params['svg_icons_options'] = $svg_icons_options;
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['product_page'] = $this->get_admin_fields__button($params);
        $insert_fields_after_name['product_page'] = 'begin_product_page_div';

        $params = [];
        $params['section_title'] ="Search &amp; category page \"Add to WishList\" buttons.";
        $params['option_suffix'] = 'product_loop';
        $params['position_title'] = "Position of \"Add to WishList\" on the search and category page";
        $params['button_position_options'] = $button_position_options__product_loop;
        $params['button_shortcode'] = $button_shortcode__product_loop;
        $params['button_style_options'] = $button_style_options;
        $params['svg_icons_options'] = $svg_icons_options;
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['product_loop'] = $this->get_admin_fields__button($params);
        $insert_fields_after_name['product_loop'] = 'begin_product_loop_div';

        $params = [];
        $params['section_title'] ="Shopping cart \"Add to WishList\" button.";
        $params['option_suffix'] = 'cart';
        $params['position_title'] = "Position of \"Add to WishList\" on the shopping cart page";
        $params['button_position_options'] = $button_position_options__cart;
        $params['button_shortcode'] = $button_shortcode__cart;
        $params['button_style_options'] = $button_style_options;
        $params['svg_icons_options'] = $svg_icons_options;
        $params['default_values'] = $default_values;
        $params['omit_fields'] = ['browse_wishlist_text__','item_alread_added_text__','item_added_text__'];
        $section_admin_fields_for_insert['cart'] = $this->get_admin_fields__button($params);
        $insert_fields_after_name['cart'] = 'begin_cart_div';

        /**
         * WishList homepage
         */
        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_wishlist_homepage',
            'title'  => 'WishList Homepage',
            'section_title'  => "WishList homepage",
            'section_tab_css'  => $section_tab_css,
            'fields' => array(
                array(
                    'type'    => 'content',
                    'content' => "Description",
                    'description' => "These settings control your store's WishList homepage. Your \"WishList\" top menu links to this WishList homepage.
                    The WishList homepage has 3 buttons &amp; provides customers a quick way to 1) create WishLists, 2) find WishLists, or 3) manage their own WishLists.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'name'    => 'wl_hp_url',
                    'type'    => 'html',
                    'html' => "
                                <div class='exopite-sof-field exopite-sof-field-text no-border-bottom '>
                                    <h4 class='exopite-sof-title'>
                                        URL to your store's WishList homepage
                                        <p class='exopite-sof-description'>If you have a custom menu for your store, use this url to add the WishList Homepage to it.</p>
                                    </h4>
                                    <div class='exopite-sof-fieldset'>
                                        <span class='wishlistdotcom_woo-code-display'>".esc_html( $wl_hp_url )."</span>
                                    </div>
                                    <div class='clearfix'></div>
                                </div>
                                ",
                ),

                array(
                    'id'          => 'wishlist_hp_title',
                    'type'        => 'text',
                    'title'          => "WishList homepage title",
                    'description'   => "This is the title of your WishList homepage. This is also the title of the top menu item that leads to your WishList homepage",
                    'default'   => $default_values['wishlist_hp_title'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom '
                ),

                //store logo
                array(
                    'id'          => 'hp_banner_img',
                    'type'        => 'image',
                    'title'          => "WishList homepage banner image",
                    'default'   => $default_values['hp_banner_img'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom'
                ),
                array(
                    'id'          => 'hp_subtitle',
                    'type'        => 'text',
                    'title'          => "WishList homepage subtitle",
                    'default'   => $default_values['hp_subtitle'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom'
                ),
                array(
                    'id'          => 'hp_subtitle_font_size',
                    'name'          => 'hp_subtitle_font_size',
                    'type'        => 'select',
                    'title'       => 'Subtitle font size',
                    'attributes' => array(
                        'id' => 'hp_subtitle_font_size',
                    ),
                    'options' => $font_size_dropdown,
                    'default'   => $default_values['hp_subtitle_font_size'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'style'    => 'fancy',
                    'wrap_class'   => 'no-border-bottom',
                ),

                array(
                    'id'          => 'hp_description',
                    'type'        => 'textarea',
                    'title'       => 'Subtitle description',
                    'attributes'    => array(
                        'rows' => '3',
                    ),
                    'default'   => $default_values['hp_description'],
                    'class'       => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area',
                    'wrap_class'   => 'no-border-bottom'
                ),
                array(
                    'id'          => 'hp_description_font_size',
                    'name'          => 'hp_description_font_size',
                    'type'        => 'select',
                    'title'       => 'Description font size',
                    'attributes' => array(
                        'id' => 'hp_description_font_size',
                    ),
                    'options' => $font_size_dropdown,
                    'default'   => $default_values['hp_description_font_size'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'style'    => 'fancy',
                    'wrap_class'   => 'no-border-bottom',
                ),
                array(
                    'id'          => 'hp_alignment',
                    'name'          => 'hp_alignment',
                    'type'        => 'select',
                    'title'       => 'WishList homepage content alignment',
                    'description'       => 'Set the alignment of the text and buttons',
                    'attributes' => array(
                        'id' => 'hp_alignment',
                    ),
                    'options' => array(
                        'text_alignment_centered'   => 'Centered',
                        'text_alignment_left'   => 'Left aligned',
                        'text_alignment_right'   => 'Right aligned',
                    ),
                    'default'   => $default_values['hp_alignment'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'style'    => 'fancy',
                    'wrap_class'   => 'no-border-bottom',
                ),
                array(
                    'id'          => 'hp_footer',
                    'name'          => 'hp_footer',
                    'type'        => 'textarea',
                    'title'       => 'WishList homepage footer',
                    'attributes'    => array(
                        'rows' => '5',
                    ),
                    'default'          => $default_values['hp_footer'],
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area',
                    'wrap_class' => 'no-border-bottom',
                ),
                array(
                    'id'          => 'hp_footer_font_size',
                    'name'          => 'hp_footer_font_size',
                    'type'        => 'select',
                    'title'       => 'footer font size',
                    'attributes' => array(
                        'id' => 'hp_footer_font_size',
                    ),
                    'options' => $font_size_dropdown_footer,
                    'default'   => $default_values['hp_footer_font_size'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'style'    => 'fancy',
                    'wrap_class'   => 'no-border-bottom ',
                ),

                array(
                    'id'          => 'hp_show_poweredby',
                    'type'        => 'radio',
                    'title'       => "Show \"powered by WishList.com\" in the footer",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['hp_show_poweredby'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom',
                ),

                array(
                    'type'    => 'content',
                    'content' => "WishList homepage buttons",
                    'description' => "The homepage's 3 buttons provide quick access to: 1) Create a WishList, 2) Find a WishList, 3) My WishList",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //sub tabs
                array(
                    'type'    => 'content',
                    'content' => "
                        <div class='wishlistdotcom_woo-sub-nav-button-group'>
                            <button id='wishlistdotcom_woo_sub_nav__hp_createwishlist' data-content-id='wishlistdotcom_section__hp_createwishlist' type='button' class='wishlistdotcom_woo-sub-nav active exopite-sof-sub-nav-list-item'>Create WishList button</button>
                            <button id='wishlistdotcom_woo_sub_nav__hp_findwishlist' data-content-id='wishlistdotcom_section__hp_findwishlist' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Find WishList button</button>
                            <button id='wishlistdotcom_woo_sub_nav__hp_mywishlist' data-content-id='wishlistdotcom_section__hp_mywishlist' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>My WishList button</button>
                        </div>
                        ",
                    'description' => "",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top wishlistdotcom_woo-sub-nav-button-group-wrapper',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //button param inserted by get_admin_fields__hp_button()
                //begin hp_createwishlist
                array(
                    'name'    => 'begin_hp_createwishlist',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__hp_createwishlist' class='wishlistdotcom_woo-no-border-top--off wishlistdotcom_woo-show_hide_section__hp_createwishlist'>",
                ),
                //end hp_createwishlist
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin hp_findwishlist
                array(
                    'name'    => 'begin_hp_findwishlist',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__hp_findwishlist' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__hp_findwishlist'>",
                ),
                //end hp_findwishlist
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin hp_mywishlist
                array(
                    'name'    => 'begin_hp_mywishlist',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__hp_mywishlist' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__hp_mywishlist'>",
                ),
                //end hp_mywishlist
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //tooltip
                array(
                    'type'    => 'content',
                    'content' => 'Tooltip style',
                    'description' => 'Displays information when customers hover over the "Save to WishList" buttons',
                    'wrap_class'   => 'wishlistdotcom_woo-content-border-top exopite-sof-fieldgroup-title-h4 no-border-bottom',
                ),
                array(
                    'id'          => 'tooltip_background_color',
                    'type'        => 'color',
                    'before'      => 'Tooltip background color',
                    'default'   => $default_values['tooltip_background_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field',
                ),
                array(
                    'id'          => 'tooltip_border_color',
                    'type'        => 'color',
                    'before'      => 'Tooltip border color',
                    'default'   => $default_values['tooltip_border_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field',
                ),
                array(
                    'id'          => 'tooltip_text_color',
                    'type'        => 'color',
                    'before'      => 'Tooltip text color',
                    'default'   => $default_values['tooltip_text_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field',
                ),

                array(
                    'id'          => 'tooltip_show_shadow',
                    'type'        => 'radio',
                    'title'       => "Give the tooltip a shadow",
                    'options' => array(
                        'yes_str'    => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['tooltip_show_shadow'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class'   => 'no-border-bottom',
                ),

                array(
                    'type'    => 'content',
                    'content' => 'Custom CSS and Javascript',
                    'description' => $custom_css_description,
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),
            )
        );

        /**
         * Add to WishList
         */
        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_add_to_wishlist',
            'title'  => 'Add to WishList',
            'section_title'  => 'Add to WishList buttons and screens',
            'section_tab_css'  => $section_tab_css,
            'fields' => array(

                array(
                    'type'    => 'content',
                    'content' => "Description",
                    'description' => "These settings control the \"Add to WishList\" buttons on your store's shopping pages. These settings also control the \"Add to WishList\" screens",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //sub tabs
                array(
                    'type'    => 'content',
                    'content' => "
                        <div class='wishlistdotcom_woo-sub-nav-button-group'>
                            <button id='wishlistdotcom_woo_sub_nav__product_page' data-content-id='wishlistdotcom_section__product_page' type='button' class='wishlistdotcom_woo-sub-nav active exopite-sof-sub-nav-list-item'>Product page</button>
                            <button id='wishlistdotcom_woo_sub_nav__product_loop' data-content-id='wishlistdotcom_section__product_loop' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Search &amp; Category</button>
                            <button id='wishlistdotcom_woo_sub_nav__cart'  data-content-id='wishlistdotcom_section__cart' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Shopping Cart</button>
                        </div>
                        ",
                    'description' => "",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top wishlistdotcom_woo-sub-nav-button-group-wrapper',
                    'description_class'     => '',
                ),

                //begin product page div
                array(
                    'name'    => 'begin_product_page_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__product_page' class='wishlistdotcom_woo-show_hide_section__product_page'>",
                ),
                //end product page div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin product loop div
                array(
                    'name'    => 'begin_product_loop_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__product_loop' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__product_loop'>",
                ),
                //end product loop div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin cart div
                array(
                    'name'    => 'begin_cart_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__cart' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__cart'>",
                ),
                //end cart div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //pop window/main window
                array(
                    'type'    => 'content',
                    'content' => '"Add to WishList" screens',
                    'description' => "Your customers see these screens after clicking an Add to WishList button on your store." ,
                    'wrap_class'   => 'wishlistdotcom_woo-content-border-top exopite-sof-fieldgroup-title-h4 no-border-bottom',
                ),
                array(
                    'id'          => 'add2wishlist_window',
                    'type'        => 'radio',
                    'title'       => "Choose what happens when Add to WishList buttons are clicked",
                    'description' => "How the Add to WishList screens are presented to your customers after clicking the Add to WishList buttons",
                    'options' => array(
                        'main_window'   => "Navigate to the Add to WishList screens in the main window. The customer is returned to the original page after adding their Wish.",
                        'popwindow'   => "Open a smaller pop-window for the Add to WishList screens. This pop-window is closed after adding their Wish.",
                    ),
                    'default'   => $default_values['add2wishlist_window'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),
                array(
                    'id'          => 'show_add2wishlist_button_spinner',
                    'type'        => 'radio',
                    'title'       => "Show a spinner when the Add to WishList button is clicked",
                    'description' => "A standard circular spinner that lets customers know their Wish is being saved.",
                    'options' => array(
                        'yes_str'   => "YES, show a spinner",
                        'no_str'   => "NO, don't show a spinner",
                    ),
                    'default'   => $default_values['show_add2wishlist_button_spinner'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),
                array(
                    'id'          => 'add2wishlist_button_spinner_color',
                    'type'        => 'color',
                    'before'      => 'Add to WishList button spinner color',
                    'default'   => $default_values['add2wishlist_button_spinner_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__button_spinner',
                ),
                array(
                    'id'          => 'add2wishlist_button_spinner_height',
                    'type'        => 'text',
                    'before'      => 'Height of the Add to WishList button spinner',
                    'default'   => $default_values['add2wishlist_button_spinner_height'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__button_spinner',
                ),

                array(
                    'type'    => 'content',
                    'content' => 'Custom CSS and Javascript',
                    'description' => $custom_css_description,
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

            )
        );

        $menu_items = json_decode($this->options_all['wlcom_store_top_menu_links_json'],true);
        $menu_items_tr_html = '';
        foreach ($menu_items as $title=>$url){
            $menu_items_tr_html .= "
                <tr>
                    <td class=''><input class='wishlistdotcom_woo-form-control' type='text' name='wishlist-dot-com-for-woocommerce[wlcom_wishlist_menu_item_title][]' value='".$title."'></td>
                    <td class='wishlistdotcom_woo-table-td-md-width'><input class='wishlistdotcom_woo-form-control' type='text' name='wishlist-dot-com-for-woocommerce[wlcom_wishlist_menu_item_url][]' value='".$url."'></td>
                    <td><a class='wishlistdotcom_woo-tr-delete wishlistdotcom_woo-alert-error-color' href='javasript:void(0)'>del</a>&nbsp;&nbsp;<a class='wishlistdotcom_woo-tr-up' href='javasript:void(0)'>up</a>&nbsp;&nbsp;<a class='wishlistdotcom_woo-tr-down' href='javasript:void(0)'>down</a></td>
                </tr>
            ";
        }
        /**
         * WishList display
         */
        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_wishlist_display_template',
            'title'  => 'WishList Display',
            'section_title'  => "WishList.com display",
            'section_tab_css'  => $section_tab_css,
            'fields' => array(

                array(
                    'type'    => 'content',
                    'content' => "Description",
                    'description' => "Your WishList is powered by WishList.com. These settings control your WishList's design and functionality that's provided by WishList.com. These settings make WishList.com's functionality match your store's look,
                    so it's a seemless experience for your customers.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'type'    => 'content',
                    'content' => "WishList features &amp; layout",
                    'description' => "Customize your WishList display and management pages. ",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'id'          => 'wlcom_allow_multiple_wishlists',
                    'type'        => 'radio',
                    'title'       => "Allow customers to create multiple WishList",
                    'description' => "Customers can create multiple profiles with multiple WishLists under each profile. Like a profile for each child with WishLists under each profile.",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['wlcom_allow_multiple_wishlists'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'id'          => 'wlcom_wishlist_layout',
                    'type'        => 'select',
                    'title'       => 'Default WishList Layout',
                    'description'       => 'Set the default layout of Wishes within a WishList. Customers can then select their preferred layout.',
                    'attributes' => array(
                        'id' => 'wlcom_wishlist_layout',
                    ),
                    'options' => array(
                        'wishlist_layout_grid'   => 'Grid (rows and columns)',
                        'wishlist_layout_list'    => 'List (one item per row)',
                        'wishlist_layout_collage'    => 'Collage (a masonry layout)',
                    ),
                    'default'   => $default_values['wlcom_wishlist_layout'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'style'    => 'fancy',
                    'wrap_class'   => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_allow_profile_image',
                    'type'        => 'radio',
                    'title'       => "Allow customers to use their own profile image.",
                    'description' => "Their profile image comes from their google/apple/facebook login or their WishList.com profile",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['wlcom_allow_profile_image'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'id'          => 'wlcom_allow_wishlist_banner',
                    'type'        => 'radio',
                    'title'       => "Allow customers to decorate their WishList with a banner image at the top.",
                    'description'      => "Customers can upload thier own banner image to WishList.com as part of their profile.",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['wlcom_allow_wishlist_banner'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),
                array(
                    'id'          => 'wlcom_wishlist_header_color',
                    'type'        => 'color',
                    'title'      => "Default color of your customers' WishList banner",
                    'description'      => "The WishList banner is a section at the top of the WishList with the customer's name and profile image",
                    'default'   => $default_values['wlcom_wishlist_header_color'],
                    'class'     => '',
                    'wrap_class'   => 'no-border-bottom',
                ),
                array(
                    'id'          => 'wlcom_wishlist_header_font_color',
                    'type'        => 'color',
                    'title'      => "Text color of customer's name within their WishList's banner section",
                    'default'   => $default_values['wlcom_wishlist_header_font_color'],
                    'class'     => '',
                    'wrap_class'   => 'no-border-bottom',
                ),
                array(
                    'id'          => 'wlcom_empty_wishlist_prompt',
                    'type'        => 'textarea',
                    'title'       => "Message to show the customer when their WishList is empty",
                    'attributes'    => array(
                        'rows' => '3',
                    ),
                    'default'          => $default_values['wlcom_empty_wishlist_prompt'],
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area-html',
                    'wrap_class' => 'no-border-bottom',
                ),
                array(
                    'id'          => 'wlcom_empty_wishlist_shop_button',
                    'type'        => 'text',
                    'title'          => "Shop button for empty WishLists",
                    'default'   => $default_values['wlcom_empty_wishlist_shop_button'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom'
                ),
                array(
                    'id'          => 'wlcom_wishlist_icon_background_color__gift',
                    'type'        => 'color',
                    'title'      => "Background color of WishList related icons (SVGs)",
                    'default'   => $default_values['wlcom_wishlist_icon_background_color__gift'],
                    'class'     => '',
                    'wrap_class'   => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_profile_initials_background_color',
                    'type'        => 'color',
                    'title'      => "Background color for profile initials",
                    'description'      => "For when a profile image is not used or available, and the customer's initials are used as a profile icon instead.",
                    'before'      => "Leave blank to use a variety of subtle colors",
                    'default'   => $default_values['wlcom_profile_initials_background_color'],
                    'class'     => '',
                    'wrap_class'   => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_allow_follow',
                    'type'        => 'radio',
                    'title'       => "Allow following. People can follow a WishList",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['wlcom_allow_follow'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'id'          => 'wlcom_allow_friends',
                    'type'        => 'radio',
                    'title'       => "Allow friending. People can request to be friends. Friend requests can then be accepted or denied",
                    'description' => "Customers can disable friending on their WishLists, if they want.",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['wlcom_allow_friends'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'id'          => 'wlcom_allow_comments',
                    'type'        => 'radio',
                    'title'       => "Allow comments. Friends can comment on each other's WishLists.",
                    'description' => "Customers can disable commenting on their WishLists, if they want.",
                    'options' => array(
                        'yes_str'   => "Yes",
                        'no_str'   => "No",
                    ),
                    'default'   => $default_values['wlcom_allow_comments'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'type'    => 'content',
                    'content' => "WishList template header &amp; footer",
                    'description' => "This controls the header, top menu and footer that's used when displaying your WishList on WishList.com. The header is meant to look like your store's header.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'id'          => 'wlcom_wishlist_template_use_html__header',
                    'type'        => 'radio',
                    'title'       => "Select the source of your WishList template header.",
                    'options' => array(
                        'no_str'   => "Use your store's logo, name and top menu.",
                        'yes_str'   => "Use custom html",
                    ),
                    'default'   => $default_values['wlcom_wishlist_template_use_html__header'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'id'          => 'wlcom_header_template_sticky',
                    'type'        => 'radio',
                    'title'       => "Make the header sticky so it's always showing at the top",
                    'description'       => "If you're store's top menu is always showing at the top, then select Yes.",
                    'options' => array(
                        'no_str'   => "No",
                        'yes_str'   => "Yes",
                    ),
                    'default'   => $default_values['wlcom_header_template_sticky'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_background_color',
                    'type'        => 'color',
                    'before'      => 'Header background color',
                    'default'   => $default_values['wlcom_header_template_background_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'             => 'wlcom_header_template_top_bottom_padding',
                    'type'        => 'text',
                    'before'      => 'Header top &amp; bottom padding',
                    'default'          => $default_values['wlcom_header_template_top_bottom_padding'],
                    'after'     => 'like 1em',
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_bottom_border_color',
                    'type'        => 'color',
                    'before'      => 'Header bottom border color',
                    'default'   => $default_values['wlcom_header_template_bottom_border_color'],
                    'after'     => 'to hide, leave empty',
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_bottom_border_width',
                    'type'        => 'text',
                    'before'      => 'Header bottom border width',
                    'default'   => $default_values['wlcom_header_template_bottom_border_width'],
                    'after'     => 'like '.$default_values['wlcom_header_template_bottom_border_width'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                /*array(
                    'id'          => 'wlcom_header_template_bottom_margin',
                    'type'        => 'text',
                    'before'      => 'Header bottom margin (space below)',
                    'default'   => $default_values['wlcom_header_template_bottom_margin'],
                    'after'     => 'like '.$default_values['wlcom_header_template_bottom_margin'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),*/

                array(
                    'id'          => 'wlcom_template_store_logo',
                    'type'        => 'image',
                    'title'          => "Header store logo",
                    'description' => "This goes in the top left of your WishList template header",
                    'default'   => $default_values['wlcom_template_store_logo'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header'
                ),
                array(
                    'id'          => 'wlcom_template_store_name',
                    'type'        => 'text',
                    'title'          => "Header store name",
                    'description' => "This goes to the right of your logo in your WishList template header",
                    'default'   => $default_values['wlcom_template_store_name'],
                    'class'       => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header'
                ),


                //menu html
                //begin_share_button_div
                array(
                    'name'    => 'test',
                    'type'    => 'html',
                    'html' => "
                        <div id='wishlistdotcom_woo_header_menu' class='exopite-sof-field exopite-sof-field-text no-border-bottom wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header'>
                            <h4 class='exopite-sof-title'>
                                Header menu
                                <p class='exopite-sof-description'>This is the menu in your WishList template header. It should typically match your store's top menu.</p>
                            </h4>
                            <div class='exopite-sof-fieldset'>
                                <table>
                                    ".$menu_items_tr_html."
                                    <tr>
                                        <td colspan='3' align='right'>
                                            <a class='wishlistdotcom_woo-tr-add exopite-sof-btn wishlistdotcom_woo-button-md' href='javascript:void(0)'>add menu item</a>
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </div>
                        ",
                ),

                array(
                    'id'          => 'wlcom_template_store_name__text_color',
                    'type'        => 'color',
                    'before'      => 'Header store name color',
                    'default'   => $default_values['wlcom_template_store_name__text_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_menu_item__text_color',
                    'type'        => 'color',
                    'before'      => 'Menu text color',
                    'default'   => $default_values['wlcom_header_template_menu_item__text_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_menu_item__text_color_hover',
                    'type'        => 'color',
                    'before'      => 'Menu hover text color',
                    'default'   => $default_values['wlcom_header_template_menu_item__text_color_hover'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_menu_item__text_font_size',
                    'type'        => 'text',
                    'before'      => 'Menu text font size',
                    'after'      => "like: 16px",
                    'default'   => $default_values['wlcom_header_template_menu_item__text_font_size'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),
                array(
                    'id'          => 'wlcom_header_template_menu_item__text_decoration',
                    'type'        => 'text',
                    'before'      => 'Menu text decoration',
                    'after'      => "like: none or underline" ,
                    'default'   => $default_values['wlcom_header_template_menu_item__text_decoration'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_items__header',
                ),

                array(
                    'id'          => 'wlcom_template_html__header',
                    'type'        => 'textarea',
                    'title'       => 'WishList header template custom html',
                    'description' => "This html controls the WishList template header and its top menu. It's meant to make your WishList functionality that's displayed by WishList.com look like your store. Add CSS and JS using the \"Custom CSS &amp; JS tab\"",
                    'attributes'    => array(
                        'rows' => '7',
                    ),
                    'default'          => $default_values['wlcom_template_html__header'],
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area-html',
                    'wrap_class' => 'no-border-bottom wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_html__header',
                ),
                array(
                    'id'          => 'wlcom_wishlist_template_use_html__footer',
                    'type'        => 'radio',
                    'title'       => "WishList template footer.",
                    'options' => array(
                        'no_str'   => "Don't add a WishList template footer",
                        'yes_str'   => "Use custom html",
                    ),
                    'default'   => $default_values['wlcom_wishlist_template_use_html__footer'],
                    'class'     => '',
                    'style'    => 'fancy',
                    'wrap_class' =>'no-border-bottom ',
                ),

                array(
                    'id'          => 'wlcom_template_html__footer',
                    'type'        => 'textarea',
                    'title'       => 'WishList footer template custom html',
                    'description' => "This html controls the WishList template footer. Add CSS and JS using the \"Custom CSS &amp; JS tab\"",
                    'attributes'    => array(
                        'rows' => '7',
                    ),
                    'default'          => $default_values['wlcom_template_html__footer'],
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area-html',
                    'wrap_class' => 'no-border-bottom wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wishlist_template_html__footer',
                ),

                array(
                    'type'    => 'content',
                    'content' => "WishList colors, fonts, &amp; settings",
                    'description' => "These settings are used for your WishList's text, buttons and alerts that are used to display your WishList on WishList.com",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //sub tabs
                array(
                    'type'    => 'content',
                    'content' => "
                        <div class='wishlistdotcom_woo-sub-nav-button-group'>
                            <button id='wishlistdotcom_woo_sub_nav__text'  data-content-id='wishlistdotcom_section__text' type='button' class='wishlistdotcom_woo-sub-nav active exopite-sof-sub-nav-list-item'>Text</button>
                            <button id='wishlistdotcom_woo_sub_nav__share_button' data-content-id='wishlistdotcom_section__share_button' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Share WishList button</button>
                            <button id='wishlistdotcom_woo_sub_nav__primary_button' data-content-id='wishlistdotcom_section__primary_button' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Primary button</button>
                            <button id='wishlistdotcom_woo_sub_nav__success_button' data-content-id='wishlistdotcom_section__success_button' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Success button</button>
                            <button id='wishlistdotcom_woo_sub_nav__danger_button'  data-content-id='wishlistdotcom_section__danger_button' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Danger button</button>
                            <button id='wishlistdotcom_woo_sub_nav__wish_received_flag'  data-content-id='wishlistdotcom_section__wish_received_flag' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Gift already received flag</button>
                            <button id='wishlistdotcom_woo_sub_nav__alerts'  data-content-id='wishlistdotcom_section__alerts' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Alerts</button>
                        </div>
                        ",
                    'description' => "",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top wishlistdotcom_woo-sub-nav-button-group-wrapper',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //begin_text_div
                array(
                    'name'    => 'begin_gift_text_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__text' class='wishlistdotcom_woo-show_hide_section__text'>",
                ),
                array(
                    'id'          => 'wlcom_text_color',
                    'type'        => 'color',
                    'before'      => 'Default text color',
                    'default'   => $default_values['wlcom_text_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                array(
                    'id'          => 'wlcom_font_size',
                    'type'        => 'text',
                    'before'      => 'Default font size',
                    'default'   => $default_values['wlcom_font_size'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                array(
                    'id'          => 'wlcom_font_family',
                    'type'        => 'text',
                    'before'      => 'Default font family',
                    'default'   => $default_values['wlcom_font_family'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                //end_text_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin_share_button_div
                array(
                    'name'    => 'begin_share_button_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__share_button' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__share_button'>",
                ),
                //end begin_share_button_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin_primary_button_div
                array(
                    'name'    => 'begin_primary_button_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__primary_button' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__primary_button'>",
                ),
                //end begin_primary_button_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin_success_button_div
                array(
                    'name'    => 'begin_success_button_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__success_button' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__success_button'>",
                ),
                //end begin_success_button_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin_danger_button_div
                array(
                    'name'    => 'begin_danger_button_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__danger_button' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__danger_button'>",
                ),
                //end begin_danger_button_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin_wish_received_flag
                array(
                    'name'    => 'begin_wish_received_flag',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__wish_received_flag' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wish_received_flag'>",
                ),
                array(
                    'id'          => 'wlcom_gift_received_button_background_color',
                    'type'        => 'color',
                    'before'      => '"Gift Already Received" flag color',
                    'default'   => $default_values['wlcom_gift_received_button_background_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                //end begin_gift_received_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                //begin_alerts_div
                array(
                    'name'    => 'begin_gift_alerts_div',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__alerts' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__alerts'>",
                ),
                array(
                    'id'          => 'wlcom_success_alert_background_color',
                    'type'        => 'color',
                    'before'      => 'Success alert background color',
                    'default'   => $default_values['wlcom_success_alert_background_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                array(
                    'id'          => 'wlcom_danger_alert_background_color',
                    'type'        => 'color',
                    'before'      => 'Danger alert background color',
                    'default'   => $default_values['wlcom_danger_alert_background_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                array(
                    'id'          => 'wlcom_info_alert_background_color',
                    'type'        => 'color',
                    'before'      => 'Info alert background color',
                    'default'   => $default_values['wlcom_info_alert_background_color'],
                    'class'     => 'wishlistdotcom_woo-form-control',
                    'wrap_class'   => 'no-border-bottom wishlistdotcom_woo-inline-field
                    ',
                ),
                //end_alerts_div
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),
                array(
                    'type'    => 'content',
                    'content' => 'Custom CSS and Javascript',
                    'description' => $custom_css_description,
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),
            )
        );

        //add commone buttons using get_admin_fields__button
        $params = [];
        $params['section_title'] ="Share WishList button colors";
        $params['option_suffix'] = 'share_button';
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['share_button'] = $this->get_admin_fields__wishlist_page_buttons($params);
        $insert_fields_after_name['share_button'] = 'begin_share_button_div';

        $params = [];
        $params['section_title'] ="Primary button colors";
        $params['option_suffix'] = 'primary_button';
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['primary_button'] = $this->get_admin_fields__wishlist_page_buttons($params);
        $insert_fields_after_name['primary_button'] = 'begin_primary_button_div';

        $params = [];
        $params['section_title'] ="Success button colors";
        $params['option_suffix'] = 'success_button';
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['success_button'] = $this->get_admin_fields__wishlist_page_buttons($params);
        $insert_fields_after_name['success_button'] = 'begin_success_button_div';

        $params = [];
        $params['section_title'] ="Danger button colors";
        $params['option_suffix'] = 'danger_button';
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['danger_button'] = $this->get_admin_fields__wishlist_page_buttons($params);
        $insert_fields_after_name['danger_button'] = 'begin_danger_button_div';

        //generate homepage button fields
        $params = [];
        //$params['section_title'] ="\"Create a WishList\" homepage button.";
        $params['section_title'] = "";
        $params['button_name'] ="Create a WishList";
        $params['option_suffix'] = 'hp_createwishlist';
        $params['button_style_options'] = $button_style_options;
        $params['svg_icons_options'] = $svg_icons_options;
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['hp_createwishlist'] = $this->get_admin_fields__hp_button($params);
        $insert_fields_after_name['hp_createwishlist'] = 'begin_hp_createwishlist';

        $params = [];
        //$params['section_title'] ="\"Find WishList\" button.";
        $params['section_title'] = "";
        $params['button_name'] ="Find a WishList";
        $params['option_suffix'] = 'hp_findwishlist';
        $params['button_style_options'] = $button_style_options;
        $params['svg_icons_options'] = $svg_icons_options;
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['hp_findwishlist'] = $this->get_admin_fields__hp_button($params);
        $insert_fields_after_name['hp_findwishlist'] = 'begin_hp_findwishlist';

        $params = [];
        //$params['section_title'] ="\"My WishList\" button.";
        $params['section_title'] = "";
        $params['button_name'] ="My WishList";
        $params['option_suffix'] = 'hp_mywishlist';
        $params['button_style_options'] = $button_style_options;
        $params['svg_icons_options'] = $svg_icons_options;
        $params['default_values'] = $default_values;
        $section_admin_fields_for_insert['hp_mywishlist'] = $this->get_admin_fields__hp_button($params);
        $insert_fields_after_name['hp_mywishlist'] = 'begin_hp_mywishlist';

        /**
         * Emails
         */
        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_emails',
            'title'  => 'Emails',
            'section_title'  => 'WishList.com email',
            'section_tab_css'  => $section_tab_css,
            'fields' => array(

                array(
                    'type'    => 'content',
                    'name'    => 'email_description',
                    'content' => "Description",
                    'description' => "These settings control WishList emails sent to your customers by WishList.com. 
                    These emails come from WishList.com and support your customers' WishList management.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //sub tabs
                array(
                    'type'    => 'content',
                    'content' => "
                        <div class='wishlistdotcom_woo-sub-nav-button-group'>
                            <button id='wishlistdotcom_woo_sub_nav__email_template' data-content-id='wishlistdotcom_section__email_template' type='button' class='wishlistdotcom_woo-sub-nav active exopite-sof-sub-nav-list-item'>Email Template</button>
                            <button id='wishlistdotcom_woo_sub_nav__welcome_email' data-content-id='wishlistdotcom_section__welcome_email' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Welcome Email</button>
                            <button id='wishlistdotcom_woo_sub_nav__empty_wishlist_reminder'  data-content-id='wishlistdotcom_section__empty_wishlist_reminder' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Empty WishList Reminder</button>
                            <button id='wishlistdotcom_woo_sub_nav__reserved_conf' data-content-id='wishlistdotcom_section__reserved_conf' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Reservation Confirmation</button>
                            <button id='wishlistdotcom_woo_sub_nav__reserved_notice'  data-content-id='wishlistdotcom_section__reserved_notice' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Wish Reserved Notice</button>
                            <button id='wishlistdotcom_woo_sub_nav__wish_on_sale'  data-content-id='wishlistdotcom_section__wish_on_sale' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Wish on Sale</button>
                            <button id='wishlistdotcom_woo_sub_nav__wish_in_stock'  data-content-id='wishlistdotcom_section__wish_in_stock' type='button' class='wishlistdotcom_woo-sub-nav exopite-sof-sub-nav-list-item'>Wish in Stock</button>
                        </div>
                        ",
                    'description' => "",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top wishlistdotcom_woo-sub-nav-button-group-wrapper',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                //begin email_template
                array(
                    'name'    => 'begin_email_template',
                    'type'    => 'html',
                    'html' => "
                            <div id='wishlistdotcom_section__email_template' class='wishlistdotcom_woo-show_hide_section__email_template'>

                            <div class='wishlistdotcom_woo-preview-email-template'>
                                <button type='button' class='wishlistdotcom_woo-button-margin-top wishlistdotcom_woo-button-md wishlistdotcom_woo-license-save-button wishlistdotcom_woo-popwin-button-js' data-button-mode='preview_email_template'>Preview welcome email</button>
                            </div>
                            ",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin welcome_email
                array(
                    'name'    => 'begin_welcome_email',
                    'type'    => 'html',
                    'html' => "
                            <div id='wishlistdotcom_section__welcome_email' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__welcome_email'>

                            <div class='wishlistdotcom_woo-preview-email-template'>
                                <button type='button' class='wishlistdotcom_woo-button-margin-top wishlistdotcom_woo-button-md wishlistdotcom_woo-license-save-button wishlistdotcom_woo-popwin-button-js' data-button-mode='preview_email_template'>Preview welcome email</button>
                            </div>
                            ",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin empty_wishlist_reminder
                array(
                    'name'    => 'begin_empty_wishlist_reminder',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__empty_wishlist_reminder' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__empty_wishlist_reminder'>",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin reserved_conf
                array(
                    'name'    => 'begin_reserved_conf',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__reserved_conf' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__reserved_conf'>",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin reserved_notice
                array(
                    'name'    => 'begin_reserved_notice',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__reserved_notice' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__reserved_notice'>",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin wish_on_sale
                array(
                    'name'    => 'begin_wish_on_sale',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__wish_on_sale' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wish_on_sale'>",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                //begin wish_in_stock
                array(
                    'name'    => 'begin_wish_in_stock',
                    'type'    => 'html',
                    'html' => "<div id='wishlistdotcom_section__wish_in_stock' class='wishlistdotcom_woo-hide wishlistdotcom_woo-show_hide_section__wish_in_stock'>",
                ),
                array(
                    'type'    => 'html',
                    'html' => "</div>",
                ),

                array(
                    'type'    => 'html',
                    'name'    => 'end_emails',
                    'html' => "",
                ),
            )
        );

        //generate button fields
        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Email Template";
        $params['option_suffix'] = 'email_template';
        $params['default_values'] = $default_values;

        $params['omit_fields'] =['wlcom_email_conf__enabl','wlcom_email_conf__subject','wlcom_email_conf__html','wlcom_email_conf__use_custom_template','wlcom_email_conf__exclude_product_categories','wlcom_email_conf__exclude_product_skus'];
        $section_admin_fields_for_insert['email_template'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['email_template'] = 'begin_email_template';

        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Welcome Email";
        $params['option_suffix'] = 'welcome_email';
        $params['default_values'] = $default_values;

        $params['omit_fields'] =['wlcom_email_conf__enable_email','wlcom_email_conf__exclude_product_categories','wlcom_email_conf__exclude_product_skus'];
        $section_admin_fields_for_insert['welcome_email'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['welcome_email'] = 'begin_welcome_email';

        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Empty WishList Reminder";
        $params['option_suffix'] = 'empty_wishlist_reminder';
        $params['default_values'] = $default_values;
        $params['omit_fields'] =['wlcom_email_conf__exclude_product_categories','wlcom_email_conf__exclude_product_skus'];

        $params['omit_fields'] =['wlcom_email_conf__enable_email','wlcom_email_conf__exclude_product_categories','wlcom_email_conf__exclude_product_skus'];
        $section_admin_fields_for_insert['empty_wishlist_reminder'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['empty_wishlist_reminder'] = 'begin_empty_wishlist_reminder';

        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Reservation Confirmation";
        $params['option_suffix'] = 'reserved_conf';
        $params['default_values'] = $default_values;

        $params['omit_fields'] =['wlcom_email_conf__enable_email','wlcom_email_conf__exclude_product_categories','wlcom_email_conf__exclude_product_skus'];
        $section_admin_fields_for_insert['reserved_conf'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['reserved_conf'] = 'begin_reserved_conf';

        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Wish Reserved Notice";
        $params['option_suffix'] = 'reserved_notice';
        $params['default_values'] = $default_values;

        $params['omit_fields'] =['wlcom_email_conf__enable_email','wlcom_email_conf__exclude_product_categories','wlcom_email_conf__exclude_product_skus'];
        $section_admin_fields_for_insert['reserved_notice'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['reserved_notice'] = 'begin_reserved_notice';

        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Wish on Sale";
        $params['option_suffix'] = 'wish_on_sale';
        $params['default_values'] = $default_values;

        $section_admin_fields_for_insert['wish_on_sale'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['wish_on_sale'] = 'begin_wish_on_sale';

        $params = [];
        $params['options'] = $this->options_all;
        $params['section_title'] ="Wish in Stock";
        $params['option_suffix'] = 'wish_in_stock';
        $params['default_values'] = $default_values;

        $section_admin_fields_for_insert['wish_in_stock'] = $this->get_admin_fields__emails($params);
        $insert_fields_after_name['wish_in_stock'] = 'begin_wish_in_stock';

        /**
         * WishList Dashboard
         */

        /**
         * Dashboard button
         */
        $WishList_dashboard_button = "
            <div class='wishlistdotcom_woo-title-subsection wishlistdotcom_woo-dashboard-prompt-div'>
                <div class='wishlistdotcom_woo-dashboard-plugin'>
                    <button type='button' class='wishlistdotcom_woo-button-margin-top wishlistdotcom_woo-button-md wishlistdotcom_woo-popwin-button-js' data-button-mode='dashboard'>Open Dashboard</button>
                </div>
            </div>

            <script>
                var wishlistdotcom_woo_env='".$store_id_a['env']."';
                var wishlistdotcom_woo_wlcom_plgn_plugin_display_status='".$wlcom_plgn_plugin_display_status."';
                var wishlistdotcom_woo_store_uuid='".$store_id_a['store_uuid']."';
                var wishlistdotcom_woo_api_domain='".$store_id_a['api_domain']."';
                var wishlistdotcom_woo_api_domain_mode='".$store_id_a['api_domain_mode']."';
                var wishlistdotcom_woo_store_url='".$store_id_a['store_url']."';
                var wishlistdotcom_woo_service_level='".$service_level."';
                var wishlistdotcom_woo_valid_api_key='".$valid_api_key."';                
            </script>

        ";

        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_dashboard',
            'title'  => 'Dashboard',
            'section_title'  => 'WishList.com dashboard',
            'section_tab_css'  => $section_tab_css,
            'fields' => array(

                array(
                    'name'    => 'wishlist_dashboard_button',
                    'type'    => 'html',
                    'html' => "<div class='wishlistdotcom_woo-section-margin-top'> </div>",
                    'wrap_class'   => 'wishlistdotcom_woo-no-border-top',
                ),

                array(
                    'type'    => 'content',
                    'content' => "Description",
                    'description' => "Your WishList data is provided by WishList.com. <br>You can view &amp; download your WishList data through your WishList.com dashboard.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'name'    => 'wishlist_dashboard_button',
                    'type'    => 'html',
                    'html' => $WishList_dashboard_button,
                    'wrap_class'   => 'wishlistdotcom_woo-no-border-top',
                ),

                array(
                    'type'    => 'content',
                    'content' => '',
                    'description' => '',
                    'wrap_class'   => 'wishlistdotcom_woo-margin-bottom no-border-bottom wishlistdotcom_woo-no-border-top',
                ),

            )
        );

        /**
         * WishList Custom CSS
         */
        $fields[] = array(
            'name'   => 'wishlistdotcom_woo_WishList_custom_css',
            'title'  => 'Custom CSS &amp; JS',
            'section_title'  => 'Custom CSS and Javascript',
            'section_tab_css'  => $section_tab_css,
            'fields' => array(

                array(
                    'type'    => 'content',
                    'content' => "Description",
                    'description' => "These settings give you a convenient way to use your own CSS and Javascript with your WishList.",
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description',
                ),

                array(
                    'id'          => 'wishlist_custom_css',
                    'type'        => 'textarea',
                    'title'       => 'WishList custom CSS',
                    'description' => 'Enter custom CSS to design the "Save to WishList.com" buttons and all other WishList elements on your store.',
                    'attributes'    => array(
                        'rows' => '7',
                    ),
                    'default'          => $default_values['wishlist_custom_css'],
                    'after'       => "Custom CSS (don't include the &lt;style&gt;&lt;/style&gt; tags)",
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area-html',
                    'wrap_class' => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wishlist_custom_js',
                    'type'        => 'textarea',
                    'title'       => 'WishList custom Javascript',
                    'description' => 'Enter custom Javascript to control WishList elements on your store.',
                    'attributes'    => array(
                        'rows' => '7',
                    ),
                    'default'          => $default_values['wishlist_custom_js'],
                    'after'       => "Custom Javascript (don't include the &lt;script&gt;&lt;/script&gt; tags)",
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area-html',
                    'wrap_class' => 'no-border-bottom',
                ),

                array(
                    'id'          => 'wlcom_wishlist_custom_css',
                    'type'        => 'textarea',
                    'title'       => "Custom CSS for your WishList that's displayed by WishList.com",
                    'description' => "This CSS controls the design of your WishList that's displayed by WishList.com.",
                    'attributes'    => array(
                        'rows' => '7',
                    ),
                    'default'          => '',
                    'after'       => "Custom CSS (don't include the &lt;style&gt;&lt;/style&gt; tags)",
                    'class'     => 'wishlistdotcom_woo-form-control wishlistdotcom_woo-text_area-html',
                    'wrap_class' => 'no-border-bottom',
                ),

                array(
                    'type'    => 'content',
                    'content' => '<a id="advanced_add2wishlist_button_template_link" href="javascript:void(0)">For advanced users</a>',
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-content-border-top wishlistdotcom_woo-advanced-add2wishlist-button-template',
                ),
                array(
                    'type'    => 'content',
                    'content' => "
                    <div>
                        <div class='wishlistdotcom_woo-content-subtitle'>You can also control the \"Save to WishList.com\" buttons and other elements using templates.</div>
                        <ol class='wishlistdotcom_woo-ol-list'>
                            <li>Copy the corresponding WishList.com template file from wp-content/plugins/wishlist-dot-com-for-woocommerce/public/templates/</li>
                            <li>Paste the template file into your theme's directory, like wp-content/themes/storefront.</li>
                            <li>Edit that template to customize the button and other elements.</li>
                        </ol>
                    </div>
                    ",
                    'attributes' => array(
                        'id' => 'advanced_add2wishlist_button_template_content',
                        'class' => 'wishlistdotcom_woo-hide'
                    ),
                    'description' => '',
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-field-tight-top-bottom-margin wishlistdotcom_woo-no-border-top
                    ',
                ),
            )
        );

        //insert fields added to $section_admin_fields_for_insert.
        foreach ($section_admin_fields_for_insert as $section=>$admin_fields){
            foreach ($fields as $field_index=>$parent_fields){
                foreach ($parent_fields['fields'] as $child_field_index=>$field){
                    //insert at the right spot
                    if(
                        (isset($field['name']) && $field['name'] == $insert_fields_after_name[$section]) ||
                        (isset($field['id']) && $field['id'] == $insert_fields_after_name[$section])
                        ){
                        array_splice($fields[$field_index]['fields'], $child_field_index+1, 0, $admin_fields);
                    }
                }
            }
        }

        /**
         * instantiate your admin page
         */
        $options_panel = new Exopite_Simple_Options_Framework( $config_submenu, $fields );

    }

    /**
     * Get html for the licensing fields
     *
     * @param $params
     * @return array
     */
    public function get_connect_to_wishlist_field($params){

        $valid_api_key = $params['valid_api_key'];
        $default_values = $params['default_values'];
        $store_id_a = $params['store_id_a'];
        $wlcom_plgn_plugin_display_status = $params['wlcom_plgn_plugin_display_status'];
        $wlcom_store_shop_url = $params['wlcom_store_shop_url'];

        /**
         * Connect Store Prompt
         */
        $permalink_structure = get_option('permalink_structure');
        $store_url_test = wc_get_page_permalink('wldc-wishlist');
        $store_url_prompt = "
            <div class='wishlistdotcom_woo-connect-store-prompt-store-path'>
                Store's URL: <strong>".$wlcom_store_shop_url."</strong>
            </div>
        ";
        if(str_contains($wlcom_store_shop_url,'?')){
            $change_store_url_prompt = "
                <div class='wishlistdotcom_woo-connect-store-prompt notice notice-error'>
                    Consider changing your store's URL to not use \"?\".
                    <br>See Wordpress Settings / Permalinks, then Permalink structure: \"Post name\", and Product permalinks: \"Shop base\".
                    <br>After changing your store's permalink URLs, connect your store to WishList.com.
                </div>
            ";
        }else{
            $change_store_url_prompt = '';
        }

        $connect_store_description = "
            <div class='wishlistdotcom_woo-title-subsection wishlistdotcom_woo-connect-store-section'>
                <div class='wishlistdotcom_woo-connect-store wishlistdotcom_woo-hide'>
                    ".$store_url_prompt."
                    ".$change_store_url_prompt."
                    <div class='wishlistdotcom_woo-connect-store-prompt notice notice-info'>
                        <p>
                            To begin, connect your store to WishList.com, for free.
                        </p>
                        <p>
                            Your store’s WishList is powered by WishList.com. Your customers will have a \"Universal WishList\" that works on your store and WishList.com.                        </p>
                        <p>
                            Your store, and its WishLists, will be available on WishList.com for people to find and share.
                        </p>
                    </div>
                    <button type='button' class='wishlistdotcom_woo-button-margin-top wishlistdotcom_woo-button-md wishlistdotcom_woo-connect-to-wldcom-button wishlistdotcom_woo-popwin-button-js' data-button-mode='connect_store'>Connect now</button>
                </div>
                <div class='wishlistdotcom_woo-connected-store wishlistdotcom_woo-hide'>
                    ".$store_url_prompt."
                    ".$change_store_url_prompt."
                    <div class='wishlistdotcom_woo-connect-store-prompt notice notice-success'>
                        Your store is connected to WishList.com. Your customers have a \"Universal WishList\" that works on your store and on WishList.com.
                    </div>
                    <div>
                        <button type='button' class='wishlistdotcom_woo-button-margin-top wishlistdotcom_woo-button-md wishlistdotcom_woo-connect-to-wldcom-button wishlistdotcom_woo-popwin-button-js' data-button-mode='connect_store'>Review Connection</button>
                    </div>
                    <div class='wishlistdotcom_woo-margin-top-40'>
                        <!-- <button type='button' class='wishlistdotcom_woo-button-margin-top wishlistdotcom_woo-button-md wishlistdotcom_woo-btn-danger wishlistdotcom_woo-connect-to-wldcom-button wishlistdotcom_woo-popwin-button-js' data-button-mode='connect_store'>Delete Connection</button> -->
                        <a class='wishlistdotcom_woo-danger-light-text wishlistdotcom_woo-popwin-button-js' href='javascript:void(0);' data-button-mode='delete_connection'>Delete Connection</a>
                    </div>
                </div>
            </div>
        ";

        $section_tab_css = '';
        $licensing_field = array(
            'name'   => $this->connect_wldcom_section_name,
            'title'  => 'Connect to WishList.com',
            'section_title'  => 'Connect to WishList.com',
            'section_tab_css'  => $section_tab_css,
            'fields' => array(

                array(
                    'type'    => 'content',
                    'content' => 'Connect your store to WishList.com',
                    'id' => 'connect_title',
                    'description' => '',
                    'wrap_class'   => 'exopite-sof-fieldgroup-title no-border-bottom wishlistdotcom_woo-no-border-top',
                    'description_class'     => 'wishlistdotcom_woo-content-description-licensing',
                ),

                array(
                    'name'    => 'connect_description',
                    'type'    => 'html',
                    'html' => $connect_store_description,
                ),

            )
        );

        return $licensing_field;
    }

    /**
     * Adjust options prior to saving
     *
     * @param $options
     * @return array
     */
    public function adjust_options_prior_to_saving($options){

        //see filter exopite_sof_save_menu_options
        //$options is passed by calling hook

        //get existing options, which is a merge of all options saved and all default options
        $existing_options = $this->get_options('all');
        //merged existing options with the options just submitted ($options)
        $options__all = array_merge($existing_options, $options);

        //gathered by wishlist-dot-com-for-woocommerce-admin.js via iframe load of wishlist homepage
        //populate colors from hidden fields if this is the first time colors are being saved
        //hidden fields are set by loading WishList hp template in a hidden iframe and using js to get the colors of the default button
        if( isset($existing_options['primary_colors_saved']) && isset($options['primary_colors_saved']) && $existing_options['primary_colors_saved'] == 'no_str' && $options['primary_colors_saved'] == 'yes_str'){
            if(!empty($options['icon_src'])){
                $options__all['wlcom_plgn_your_store_icon'] = $options['icon_src'];
            }
            foreach ($options__all as $key=>$val){
                $skip = false;
                if(str_contains($key,'_hover_')){
                    $source_mouse_mode = 'hover';
                }else{
                    $source_mouse_mode = 'regular';
                }
                if(str_contains($key,'__danger_button') && $options['primary_backgroundColor__'.$source_mouse_mode] != $options['danger_backgroundColor__'.$source_mouse_mode]){
                    $source_color_class = 'danger';
                }elseif(str_contains($key,'__danger_button') && $options['primary_backgroundColor__'.$source_mouse_mode] == $options['danger_backgroundColor__'.$source_mouse_mode]){
                    //danger is same color as primary
                    $skip = true;
                }elseif(str_contains($key,'__success_button')){
                    //we don't get this color as it doesn't have a standard button css class
                    $skip = true;
                }else{
                    $source_color_class = 'primary';
                }

                //copy color from hidden field to button
                //hidden field's name is $source_color_class.'_backgroundColor__'.$source_mouse_mode
                if($skip){
                    //do nothing
                }elseif(str_contains($key,'wishlist_button_background_')){
                    $options__all[$key] = $options[$source_color_class.'_backgroundColor__'.$source_mouse_mode];
                }elseif(str_contains($key,'wishlist_button_text_color_')){
                    $options__all[$key] = $options[$source_color_class.'_color__'.$source_mouse_mode];
                }elseif(str_contains($key,'wishlist_button_border_color_')){
                    $options__all[$key] = $options[$source_color_class.'_borderColor__'.$source_mouse_mode];
                }
            }

            //top menu styles
            //gathered by wishlist-dot-com-for-woocommerce-admin.js via iframe load of wishlist homepage
            if(isset($options['header_template_store_name_style'])){
                //store name style
                //transmitted to wishlist.com
                $options__all['wlcom_header_template_store_name_style'] = $options['header_template_store_name_style'];

                $header_template_store_name_style = json_decode($options['header_template_store_name_style'],true);
                if(isset($header_template_store_name_style['color'])){
                    $options__all['wlcom_template_store_name__text_color'] = $header_template_store_name_style['color'];
                }

                unset($options['header_template_store_name_style']);
            }
            if(isset($options['default_body_font_style'])){
                //body style
                $options__all['wlcom_default_body_font_style'] = $options['default_body_font_style'];
                $default_body_font_style = json_decode($options['default_body_font_style'], true);
                if(isset($default_body_font_style['color'])){
                    $options__all['wlcom_text_color'] = $default_body_font_style['color'];
                }
                if(isset($default_body_font_style['fontSize'])){
                    $options__all['wlcom_font_size'] = $default_body_font_style['fontSize'];
                }
                if(isset($default_body_font_style['fontFamily'])){
                    $options__all['wlcom_font_family'] = $default_body_font_style['fontFamily'];
                }

                unset($options['default_body_font_style']);
            }
        }

        //copy button style from another button
        $copy_button_style_target_source = [
            'wishlist_button_style__hp_findwishlist'=>'wishlist_button_style__hp_createwishlist',
            'wishlist_button_style__hp_mywishlist'=>'wishlist_button_style__hp_createwishlist',
            'wishlist_button_style__product_loop'=>'wishlist_button_style__product_page',
            'wishlist_button_style__cart'=>'wishlist_button_style__product_page',
        ];
        foreach ($copy_button_style_target_source as $target=>$source){
            $target_suffix = str_replace('wishlist_button_style__','', $target);
            $source_suffix = str_replace('wishlist_button_style__','', $source);

            //see if copy_style is yes
            if($options__all['copy_style__'.$target_suffix] == 'yes_str'){
                foreach ($options__all as $key=>$val){
                    //skip button settings for copy_style__, wishlist_button_position__ and wishlist_button_text__
                    if(str_contains($key,'__'.$target_suffix) &&
                        !str_contains($key,'copy_style__') && !str_contains($key, 'wishlist_button_position__') && !str_contains($key,'wishlist_button_text__') && !str_contains($key,'button_tooltip__')
                    ){
                        $options__all[$key] = $options__all[str_replace('__'.$target_suffix, '__'.$source_suffix, $key)];
                    }
                }
            }
        }

        //if featured product list has changed, then add featured products to product queue to be sent to wlcom
        $update_featured_queue = false;
        $existing_featured_product_skus_a = $existing_options['existing_featured_product_skus'];

        //deal with empty select2
        $find_key_suffix = '__hidden_select2';
        foreach ($options as $key=>$val){
            if(str_contains($key,$find_key_suffix)){
                $select2_var = str_replace($find_key_suffix,'',$key);
                if(!isset($options[$select2_var])){
                    $options__all[$select2_var] = [];
                }
                unset($options__all[$key]);
            }
        }
        $wlcom_featured_product_skus_a = $options__all['wlcom_featured_product_skus'];

        //limit to $this->featured_products_count
        if(!empty($wlcom_featured_product_skus_a)){
            $wlcom_featured_product_skus_a = array_slice($wlcom_featured_product_skus_a, 0, $this->featured_products_count);
            $options__all['wlcom_featured_product_skus'] = $wlcom_featured_product_skus_a;
        }

        //if featured product list has changed, then add featured products to product queue to be sent to wlcom
        foreach ($wlcom_featured_product_skus_a as $featured_sku){
            if(!in_array($featured_sku,$existing_featured_product_skus_a)){
                $update_featured_queue = true;
            }
        }

        if($update_featured_queue){
            //get product ids from skus
            foreach ($wlcom_featured_product_skus_a as $item){
                $item_a = explode('^',$item);
                $product_sku = $item_a[0];
                $product_id = wc_get_product_id_by_sku($product_sku);

                if(!empty($product_id)){
                    $this->add_productid_to_wlcom_queue($product_id);
                }
            }
        }
        //save current wlcom_featured_product_skus for comparison next time
        $options__all['existing_featured_product_skus'] = $options__all['wlcom_featured_product_skus'];

        //process header template menu items
        $header_menu_items = [];
        //if top menu items passed, save to wlcom_store_top_menu_links_json.
        if (isset($options__all['wlcom_wishlist_menu_item_title']) && isset($options__all['wlcom_wishlist_menu_item_url'])) {
            $menu_titles = $options__all['wlcom_wishlist_menu_item_title'];
            $menu_urls = $options__all['wlcom_wishlist_menu_item_url'];
            //$wlcom_wishlist_menu_item_title is an array of values from the input fields
            foreach ($menu_titles as $key=>$menu_title) {
                if(!empty($menu_title) && !empty($menu_urls[$key])){
                    $header_menu_items[$menu_title] = $menu_urls[$key];
                }
            }

            $options__all['wlcom_store_top_menu_links_json'] = wp_json_encode($header_menu_items);
            unset($options__all['wlcom_wishlist_menu_item_title']);
            unset($options__all['wlcom_wishlist_menu_item_url']);
        }

        $wl_hp_page_id = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION);
        if ( empty($wl_hp_page_id) ) {
            //Just in case. This is typically done as part of the Wishlist_Dot_Com_For_Woocommerce_Activator class.
            wc_create_page(
                sanitize_title_with_dashes( WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_TITLE ),
                WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION,
                WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_TITLE,
                WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_CONTENT
            );

            //now add wl hp page to top menu json. note that this is after wlcom_store_top_menu_links_json above.
            $wl_hp_page_id = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION);
            $wl_hp_page_title = get_the_title($wl_hp_page_id);
            $wl_hp_url = get_permalink($wl_hp_page_id);
            $menu_items = json_decode($this->options_all['wlcom_store_top_menu_links_json'],true);
            if(!array_key_exists($wl_hp_page_title,$menu_items)){
                $menu_items[$wl_hp_page_title] = $wl_hp_url;
                $options__all['wlcom_store_top_menu_links_json'] = wp_json_encode($menu_items);
            }
        }

        //save wishlist homepage url
        $wl_hp_page_id = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION);
        $wl_hp_url = get_permalink($wl_hp_page_id);
        $options__all['wlcom_wl_homepage_url'] = $wl_hp_url;

        //trim options to avoid expansion due to merge of existing with new
        $final_options = [];
        $skipped_options = [];
        $default_values = $this->get_field_defaults__options('all');
        foreach ($options__all as $key=>$val){
            //for security reason, limit final options to those with a default value. This means all options, including "special keys", must have a default.
            if(isset($default_values[$key])){
                $final_options[$key] = $val;
            }else{
                $skipped_options[$key] = $val;
            }
        }

        //we do this here since there's no hook that passes the filtered options adjusted by this function
        $this->save_special_options($final_options);

        return $final_options;
    }

    /**
     * Save special options to their own field in the db
     *
     * @param $options
     * @return void
     */
    public function save_special_options($options){

        //see filter exopite_sof_save_menu_options
        //$options is passed by calling hook

        if(!empty($options['delete_store_uuid'])){
            $store_uuid = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_store_uuid');
            if($store_uuid == $options['delete_store_uuid']){
                //connected store_uuid was deleted on WishList.com. set option vals to empty string to be saved below
                $options['store_uuid'] = '';
                $options['api_key'] = '';
                $options['valid_api_key'] = 'no_str';
                $options['store_api_subdomain'] = '';
            }
        }

        //special keys that are saved in their own fields, and never updated
        //api_key is create by a message from wlcom popwindow for getting license key
        //see wishlistdotcom_woo_trait. 'api_key','store_uuid'
        $special_keys = $this->special_option_keys;
        foreach ($special_keys as $special_key){
            //save keys, like api_key, in special field outside of exopite options field
            //api_key is created via wishlist-dot-com-for-woocommerce-admin.js from window message from wishlist.com
            $special_key_name = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_'.$special_key;
            if(isset($options[$special_key])){
                $val = esc_html($options[$special_key]);
                if($special_key == 'store_api_subdomain'){
                    $val = cleanSubdomainString($val);
                }
                update_option($special_key_name, $val);
            }
        }

        $wl_hp_page_id = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION);
        $wl_hp_page_title = get_the_title($wl_hp_page_id);

        //wishlist_hp_title
        if(isset($options['wishlist_hp_title']) && $options['wishlist_hp_title'] != $wl_hp_page_title){
            $new_title = $options['wishlist_hp_title']; // Replace with your desired title
            $post_data = array(
                'ID'           => $wl_hp_page_id,
                'post_title'   => $new_title,
            );
            wp_update_post( $post_data );
        }

        //separate groups of fields to limit number of options necessary for the store's ui
        $find_email_conf = 'wlcom_email_conf__';
        $find_wlcom_plgn = 'wlcom_plgn_';
        $find_wlcom = 'wlcom_';
        $email_conf_a = [];
        $options_a = [];
        foreach ($options as $key=>$val){
            if(substr($key,0,strlen($find_email_conf)) == $find_email_conf){
                //email configs
                $email_conf_a[$key] = $val;
            }elseif(substr($key,0,strlen($find_wlcom_plgn)) == $find_wlcom_plgn){
                //fields transmitted to wishlist.com that are also used in store ui
                $options_a[$key] = $val;
            }elseif(substr($key,0,strlen($find_wlcom)) == $find_wlcom){
                //fields being transmitted to wishlist.com and only used on wishlist.com
                //we'll get these from all options as necessary
            }else{
                $options_a[$key] = $val;
            }
        }
        $special_key_name = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_email_conf';
        update_option($special_key_name, $email_conf_a);

        //save remaining options for store ui.
        $special_key_name = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_options';
        update_option($special_key_name, $options_a);

        //generate buttons_css and add it to options
        $buttons_css = $this->get_button_style($options);
        $buttons_css_key = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_global_buttons_css';
        update_option($buttons_css_key,$buttons_css);

        //store_uuid, api_key, valid_api_key
        //use params that are about to be saved to call wishlistdotcom_woo_store_id(). Do this instead of trying to update the
        //db and then calling wishlistdotcom_woo_store_id()
        $params = [];
        if(isset($options['store_uuid'])){
            $params['store_uuid'] = $options['store_uuid'];
        }
        if(isset($options['api_key'])){
            $params['api_key'] = $options['api_key'];
        }
        if(isset($options['valid_api_key'])){
            $params['valid_api_key'] = $options['valid_api_key'];
        }
        if(isset($options['store_api_subdomain'])){
            $params['store_api_subdomain'] = cleanSubdomainString($options['store_api_subdomain']);
        }
        $store_id_a = wishlistdotcom_woo_store_id($params);

        //Generat js and save for next time. This saves about .5ms
        //get url for wishlist homepage
        $wl_hp_page_id = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION);
        $wl_hp_url = get_permalink($wl_hp_page_id);

        $wlcom_plgn_plugin_display_status = esc_html( $options['wlcom_plgn_plugin_display_status'] );

        $icon_url = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_URL.'public/images/WishListIcon-trans-48.png';
        $add2wishlist_window = esc_html( $options['add2wishlist_window'] );
        $show_add2wishlist_button_spinner = esc_html( $options['show_add2wishlist_button_spinner'] );

        $global_js_val = "
            var wishlistdotcom_woo_env='".esc_html( $store_id_a['env'] )."';
            var wishlistdotcom_woo_plugin_display_status='".esc_html( $wlcom_plgn_plugin_display_status )."';
            var wishlistdotcom_woo_store_uuid='".esc_html( $store_id_a['store_uuid'] )."';
            var wishlistdotcom_woo_store_api_subdomain='".esc_html( $store_id_a['store_api_subdomain'] )."';
            var wishlistdotcom_woo_api_domain='".esc_html( $store_id_a['api_domain'] )."';
            var wishlistdotcom_woo_api_domain_mode='".esc_html( $store_id_a['api_domain_mode'] )."';
            var wishlistdotcom_woo_store_url='".esc_html( $store_id_a['store_url'] )."';
            var wishlistdotcom_woo_wl_hp_url='".esc_html( $wl_hp_url )."';
            var wishlistdotcom_woo_wl_hp_page_id='".esc_html( $wl_hp_page_id )."';
            var wishlistdotcom_woo_add2wishlist_window='".esc_html( $add2wishlist_window )."';
            var wishlistdotcom_woo_wl_icon_url='".esc_html( $icon_url )."';
            var wishlistdotcom_woo_show_add2wishlist_button_spinner='".esc_html( $show_add2wishlist_button_spinner )."';
            ";

        //all user inputed data is escaped by this function
        $global_js_key = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_global_js';
        update_option($global_js_key,$global_js_val);
    }

    /**
     * Post necessary options to WishList.com's api. Options are used to configure WishList.com's API for this store
     *
     * @return void
     */
    public function post_options_to_wlcom(){

        //transmit options
        $options = $this->options_all;

        //get plugin status
        $wlcom_plgn_plugin_display_status = $options['wlcom_plgn_plugin_display_status'];

        //add extra data as necessary
        //currency
        $options['wlcom_currency'] = get_woocommerce_currency();

        //only transmit what's necessary, using wlcom_ var prefix
        $options_wlcom = [];
        foreach ($options as $var=>$val){
            if(strpos($var,'wlcom_')===0){
                $options_wlcom[$var] = $options[$var];
            }
        }

        $options_wlcom['wlcom_slug'] = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG;
        $options_wlcom['wlcom_version'] = $this->version;
        $options_wlcom['wlcom_update_source'] = 'wl';

        //options
        $params['data'] = $options_wlcom;
        $params['api_action'] = 'save_options';
        $response = wishlistdotcom_woo_api_call($params);

        //transmit updated products to sync price and inventory status
        $this->transmit_queued_products_wlcom();

    }

    /**
     * Post options called by ajax to transmit to WishList.com API asynchronously
     *
     * @return void
     */
    public function post_options_to_wlcom_rec_ajax(){

        if(wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'post_options_to_wlcom_rec_ajax' ) ){

            $this->post_options_to_wlcom();

            $result = array('message' => 'Saved');
            wp_send_json($result);

        }else {
            die( 'Security check failed!' );
        }

        // Always exit to prevent extra output
        wp_die();
    }

    /**
     * Call WishList.com API to get store's serveice level. Update option in DB
     *
     * @return mixed|string
     */
    public function check_wlcom_plugin_service_level()
    {
        //options
        $params['data'] = 'empty';
        $params['api_action'] = 'check_service_level_wlcom_for_woocom';
        $responce_a = wishlistdotcom_woo_api_call($params);

        //only update if there was a good response. if not, return existing vals
        if(isset($responce_a['error'])){
            $service_level_key = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_service_level';
            $service_level = get_option($service_level_key);
            if($service_level != 'premium'){
                $service_level = 'free';
            }
            $valid_api_key_keyname = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_valid_api_key';
            $valid_api_key = get_option($valid_api_key_keyname);
            if($valid_api_key != 'yes_str'){
                $service_level = 'no_str';
            }
            $return_a = [
                'service_level'=>$service_level,
                'valid_api_key'=>$valid_api_key,
                'error'=>$responce_a['error'],
            ];
            return $return_a;
        }

        //continue
        $service_level_a = $responce_a;

        if(isset($service_level_a['service_level'])){
            $service_level = $service_level_a['service_level'];
        }else{
            $service_level = 'free';
        }
        if(!empty($service_level_a['valid_api_key'])){
            $valid_api_key = 'yes_str';
        }else{
            $valid_api_key = 'no_str';
        }

        $service_level_key = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_service_level';
        update_option($service_level_key, $service_level);

        $valid_api_key_keyname = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_valid_api_key';
        update_option($valid_api_key_keyname, $valid_api_key);

        $return_a = [
            'service_level'=>$service_level,
            'valid_api_key'=>$valid_api_key,
        ];

        return $return_a;
    }
    /**
     * A product has been updated. Add it's ID to a queue that will then be used to send updated product data to
     * WishList.com's API for things like the store's configurued promotional emails and product pricing updates
     *
     * @param $product_id
     * @return int|void
     */
    public function add_productid_to_wlcom_queue($product_id){

        //called via hooks, like woocommerce_new_product
        //we run this logic with product changes to speed up wlcom's product sync.
        //products are sync with wlcom with each wish added. but with low product wish frequency, it could take a while to sync a product

        //save product_queue that's transmitted to wishlist.com when products go on sale or are newly in stock
        $product = wc_get_product( $product_id );

        if(!$product){
            return 1;
        }

        $product_a = [];
        $is_on_sale = $product->is_on_sale();
        if($is_on_sale){
            $is_on_sale = 1;
        }elseif($is_on_sale === false){
            $is_on_sale = 0;
        }else{
            $is_on_sale = 3;
        }
        $is_in_stock = $product->is_in_stock();
        if($is_in_stock){
            $is_in_stock = 1;
        }elseif($is_in_stock === false){
            $is_in_stock = 0;
        }else{
            $is_in_stock = 3;
        }
        $product_a['onsale'] = $is_on_sale;
        $product_a['instock'] = $is_in_stock;

        //get current queue
        $wishlistdotcom_wlcom_product_queue_a = $this->get_productid_queue(['key'=>'product_queue']);

        //add product to queue
        $wishlistdotcom_wlcom_product_queue_a[$product_id]['onsale'] = $product_a['onsale'];
        $wishlistdotcom_wlcom_product_queue_a[$product_id]['instock'] = $product_a['instock'];

        //save updated queue
        $this->save_productids_to_queue(['key'=>'product_queue','wlcom_product_ids_a'=>$wishlistdotcom_wlcom_product_queue_a]);

    }

    /**
     * Get product info for dropdown via ajax
     *
     * @return json
     */
    public function get_product_droppown_rec_ajax(){

        if(wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ), 'get_product_droppown_rec_ajax' ) ){

            $return_results = [];
            $term = sanitize_text_field($_GET['term']); // Get the search term
            $mode = sanitize_text_field($_GET['mode']); // Get the mode

            if($mode == 'category'){
                $results = $this->get_matching_category_names($term);
            }elseif($mode == 'sku'){
                $results = $this->get_matching_woocommerce_skus($term);
            }
            if (!empty($results)) {
                foreach ($results as $key=>$text){
                    if($mode == 'sku'){
                        $text_tmp = $text.' ('.$key.')';
                        //concactenate sku^text so it's stored together so we don't have to do a query to display later
                        $return_results[] = ['id' => $key.'^'.$text_tmp, 'text' => $text_tmp];
                    }elseif($mode == 'category'){
                        $return_results[] = ['id' => $text, 'text' => $text];
                    }
                }
            }

            // Return results as JSON
            wp_send_json($return_results);

        }else {
            die( 'Security check failed!' );
        }

        // Always exit to prevent extra output
        wp_die();
    }

    function get_matching_category_names($search_term) {
        $args = [
            'taxonomy'     => 'product_cat',
            'name__like' => $search_term, // Search term
            'orderby'      => 'name',
            'show_count'   => 0,
            'pad_counts'   => 0,
            'hierarchical' => 0,
            'title_li'     => '',
            'hide_empty'   => 0
        ];

        $categories = get_terms($args);

        // Check for errors
        if (is_wp_error($categories)) {
            return []; // Return empty array on error
        }

        $return_categories = [];
        if(is_array($categories)){
            foreach ($categories as $cat){
                $return_categories[$cat->name] = $cat->name;
            }
        }

        return $return_categories; // Array of matching category names
    }

    function get_matching_woocommerce_skus($search_term) {
        $matching_skus = [];

        // Arguments for WP_Query
        $args = [
            'posts_per_page' => -1, // Retrieve all products that match
            'post_type'      => 'product', // Query only WooCommerce products
            'post_status'    => 'publish', // Only retrieve published products
            's'              => $search_term, // Search term
            'fields'         => 'ids', // Retrieve only IDs for performance
        ];

        // Create the WP_Query instance
        $query = new WP_Query($args);

        // Loop through posts and get SKUs
        if ($query->have_posts()) {
            while ($query->have_posts()) {
                $query->the_post();
                $product_id = get_the_ID();
                $product = wc_get_product($product_id);

                if ($product) {
                    $name = $product->get_name();
                    $name_short = substr($name,0,20);
                    $sku = $product->get_sku();
                    if (!empty($sku)) {
                        $matching_skus[$sku] = $name_short;
                    }
                }
            }
        }

        // Reset post data
        wp_reset_postdata();

        return $matching_skus;
    }

    /**
     * Get product data for a list of ids
     *
     * @return array
     */
    public function get_product_data_from_queued_product_ids($product_ids_a=[]){
        $product_data_trans = [];
        if(!empty($product_ids_a)){
            $products = wc_get_products(
                array(
                    'include' => $product_ids_a,
                    'limit'   => -1,
                )
            );
            //add to relational array
            $product_data_trans = [];
            foreach ($products as $key=>$product){
                $product_data = $this->get_product_data_wish($product);
                $product_data_trans[] = $product_data;
            }
        }
        return $product_data_trans;
    }

    /**
     * Post to queued products to WishList.com's API
     *
     * @return void
     */
    public function transmit_queued_products_wlcom(){
        //get queued product_ids
        $wishlistdotcom_wlcom_product_queue_a = $this->get_productid_queue(['key'=>'product_queue']);

        $product_ids_a = [];
        foreach ($wishlistdotcom_wlcom_product_queue_a as $product_id=>$info_a){
            $product_ids_a[] = $product_id;
        }

        $product_data_trans = [];
        if(!empty($product_ids_a)){
            $product_data_trans = $this->get_product_data_from_queued_product_ids($product_ids_a);
        }

        if(!empty($product_data_trans)){
            $params['data'] = $product_data_trans;
            $params['api_action'] = 'sync_woo_products';
            $response = wishlistdotcom_woo_api_call($params);

            //clear the queue
            $wlcom_products_key = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_product_queue';
            update_option($wlcom_products_key, '');
        }
    }

    /**
     * Reduce the size of html
     *
     * @param $str
     * @param $rem_nl
     * @return array|string|string[]|null
     */
    private function minify_html($str,$rem_nl=false){
        $search = array(
            '/\>[^\S ]+/s',
            '/[^\S ]+\</s',
            '/(\s)+/s',
            '/\> \</',
        );
        $replace = array(
            '>',
            '<',
            '\\1',
            '><',
        );
        $str = preg_replace($search, $replace, $str);
        if($rem_nl){
            $str = str_replace("\n", " ", $str);
        }
        return $str;
    }

}

