<?php
/**
 * The file that defines functions
 *
 * @link       https://www.wishlist.com/wishlist-dot-com-for-woocommerce
 * @since      1.0.0
 *
 * @package    wishlist-dot-com-for-woocommerce
 * @subpackage wishlist-dot-com-for-woocommerce/includes
 */


if ( ! function_exists( 'wishlistdotcom_woo_get_template_html' ) ) {
    /**
     * Get html from a template file.
     *
     * @param string $path   Path to get.
     * @param mixed  $var    Variables to send to template.
     * @param bool   $return Whether to return or print the template.
     *
     * @return string|void
     * @since 1.0.0
     */
    function wishlistdotcom_woo_get_template_html( $path, $var = null, $return = false ) {
        $template_path = wishlistdotcom_woo_get_template_path( $path, $var );

        if ( $var && is_array( $var ) ) {
            $atts = $var;
            extract( $var ); // phpcs:ignore WordPress.PHP.DontExtract.extract_extract
        }

        //buffer
        if ( $return ) {
            ob_start();
        }

        // output
        include $template_path;

        //return output
        if ( $return ) {
            return ob_get_clean();
        }
    }
}
if ( ! function_exists( 'wishlistdotcom_woo_get_template_path' ) ) {
    /**
     * get the templaate path
     *
     * @param string $path Path to locate.
     * @param array  $var  Unused.
     *
     * @return string
     * @since 1.0.0
     */
    function wishlistdotcom_woo_get_template_path( $path, $var = null ) {
        $woocommerce_base = WC()->template_path();

        $template_woocommerce_path = $woocommerce_base.$path;

        //theme path
        $template_path = '/'.$path;
        //plugin's default path
        if(!str_contains($path,'/')){
            $plugin_path = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_DIR.'public/templates/' . $path;
        }else{
            $plugin_path = $path;
        }

        //search for template file ($path) in theme and woocommerce locations
        $custom_template_path = locate_template(
            array(
                $template_woocommerce_path, // <theme>/woocommerce/.
                $template_path, // <theme>/.
            )
        );

        //return our plugin path, with filter
        if ( ! $custom_template_path && file_exists( $plugin_path ) ) {
            return apply_filters( 'wishlistdotcom_woo_template_path', $plugin_path, $path );
        }

        /**
         * APPLY_FILTERS: wishlistdotcom_woo_template_path
         *
         * Filter the location of the templates.
         *
         * @param string $custom_template_path Template found
         * @param string $path    Template path
         *
         * @return string
         */
        return apply_filters( 'wishlistdotcom_woo_template_path', $custom_template_path, $path );
    }
}
if ( ! function_exists( 'wishlistdotcom_woo_is_single' ) ) {
    /**
     * True if it's on a product page
     * False if in a loop
     *
     * @return bool
     * @since 1.0.0
     */
    function wishlistdotcom_woo_is_single() {
        /**
         * APPLY_FILTERS: wishlistdotcom_woo_set_is_single
         *
         * Filter whether the button is on a product page or in a loop
         *
         * @param bool $is_product True if on a product page
         *
         * @return bool
         */

        $check_is_product = (
            is_product()
            && !in_array( wc_get_loop_prop( 'name' ), array( 'related', 'up-sells' ), true )
            && ! wc_get_loop_prop( 'is_shortcode' )
        );

        return apply_filters( 'wishlistdotcom_woo_set_is_single', $check_is_product );
    }
}
if( ! function_exists('wishlistdotcom_woo_url_to_domain_path') ){
    function wishlistdotcom_woo_url_to_domain_path($url){
        //remove correct and incorrect schema
        $url_no_scheme = str_replace(['https://','http://','https','http',':'],['','','','',''],$url);
        //remove multiple slashes, like abc.com//path
        $url_no_scheme = preg_replace('#/{2,}#', '/', $url_no_scheme);
        //remove leading and trailing / and \, like /abc.com/test/
        $url_no_scheme = rtrim($url_no_scheme, '/\\');
        $url_no_scheme = ltrim($url_no_scheme, '/\\');
        $store_domain = explode('/',$url_no_scheme)[0];
        $return_a = [
            'store_domain'=>$store_domain,
            'url_no_scheme'=>$url_no_scheme,
        ];
        return $return_a;
    }
}
if(!function_exists('cleanSubdomainString')){
    function cleanSubdomainString($input) {
        // Step 1: Remove non-alphanumeric characters except for hyphens
        $processed = preg_replace('/[^a-zA-Z0-9-]/', '', $input);

        // Step 2: Replace two or more consecutive hyphens with a single hyphen
        $processed = preg_replace('/-{2,}/', '-', $processed);

        // Step 3: Remove leading and trailing hyphens
        $processed = trim($processed, '-');

        return $processed;
    }
}
if( ! function_exists('setAllowedTags') ){
    function setAllowedTags($mode='post'){
        $allowed_tags = wp_kses_allowed_html('post'); // Start with the default allowed post tags.
        if($mode == 'post_svg'){
            $allowed_tags['svg'] = array( // Add SVG and its attributes to the list.
                'class'       => true,
                'aria-hidden' => true,
                'role'        => true,
                'xmlns'       => true,
                'width'       => true,
                'height'      => true,
                'viewbox'     => true, // Note: Should be 'viewBox', but HTML attributes are lowercase in this context.
            );
            $allowed_tags['path'] = array( // Allow 'path' elements and their attributes.
                'd'    => true,
                'fill' => true,
            );
        }
        return $allowed_tags;
    }
}

