<?php

/**
 * Plugin Name:             WishList.com for WooCommerce
 * Plugin URI:              https://www.wishlist.com/wishlist-dot-com-for-woocommerce
 * Description:             WishList.com for WooCommerce is a universal WishList powered by WishList.com. Your customers' WishLists are available on your store and WishList.com. <a href="https://wishlist.com/wishlist-dot-com-for-woocommerce" target="_blank"><strong>WishList.com for WooCommerce</strong></a>
 * Version:                 3.2.5
 * Author:                  WishList.com, Inc.
 * Author URI:              https://www.wishlist.com/wishlist-dot-com-for-woocommerce
 * License:                 GPL-3.0+
 * License URI:             https://www.gnu.org/licenses/gpl-3.0.txt
 * Text Domain:             wishlist-dot-com-for-woocommerce
 * WC requires at least:    8.3
 * WC tested up to:         8.8.0
 * Requires PHP:            7.0
 *
 * @author WishList.com, Inc.
 * @package wishlist-dot-com\wishlist-dot-com-for-woocommerce
 * @version 3.2.5
*/


if ( ! defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly
//begin_variable_definitions
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_VERSION' ) ) {
    define('WISHLIST_DOT_COM_FOR_WOOCOMMERCE_VERSION', '3.2.5');
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_URL' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_URL', plugin_dir_url( __FILE__ ) );
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_DIR' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_DIR', plugin_dir_path( __FILE__ ) );
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG', 'wishlist-dot-com-for-woocommerce' );
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_TITLE' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_TITLE', 'wldc-wishlist-home');
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_CONTENT' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_CONTENT', '<!-- wp:shortcode -->[wishlistdotcom_woo_hp_shortcode]<!-- /wp:shortcode -->');
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_HOMEPAGE_PAGE_OPTION', WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_wishlist_home_page_id' );
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_TITLE' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_WISHLIST_TITLE', 'WishList' );
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_ENV' ) ) {
    define('WISHLIST_DOT_COM_FOR_WOOCOMMERCE_ENV', 'prod');
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_USE_MIN_CSS' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_USE_MIN_CSS', true );
}
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_USE_MIN_JS' ) ) {
    define( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_USE_MIN_JS', true );
}

/* begin_plugin_updater */
if ( ! defined( 'WISHLIST_DOT_COM_FOR_WOOCOMMERCE_UPDATE_URL' ) ) {
    define('WISHLIST_DOT_COM_FOR_WOOCOMMERCE_UPDATE_URL', 'https://www.wishlist.com/update_plugin');
}
/* end_plugin_updater */

//end_variable_definitions

/**
 * The code that runs during plugin deactivation.
 *
 * @return void
 */
function wishlistdotcom_woo_deactivate_plugin() {
    deactivate_plugins(plugin_basename(__FILE__));
    if (!empty($_GET['activate'])) {
        unset($_GET['activate']);
    }
}

// Ensure functions like get_plugins, etc.
require_once(ABSPATH . 'wp-admin/includes/plugin.php');

if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
    //Woocommerce is required
    add_action( 'admin_notices', 'wishlistdotcom_woo_woocommerce_admin_notice' );
    // deactivate the plugin
    add_action('admin_init', 'wishlistdotcom_woo_deactivate_plugin');
    return;
}

/**
 * Admin notice that WooCommerce is required
 *
 * @return void
 */
function wishlistdotcom_woo_woocommerce_admin_notice() {
    ?>
    <div class="error">
        <p><?php echo esc_html( "You have to install WooCommerce. \"WishList.com for WooCommerce\" requires Woocommerce." ); ?></p>
    </div>
    <?php
}

/**
 * Declare that this plugin is compatible with High-Performance Order Storage (HPOS).
 *
 * @return void
 */
add_action( 'before_woocommerce_init', function() {
    if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
    }
} );

/**
 * The code that runs during plugin activation.
 *  This action is documented in includes/class-wishlist-dot-com-for-woocommerce-activator.php
 *
 * @return void
 */
function activate_Wishlist_Dot_Com_For_Woocommerce() {
    require_once plugin_dir_path( __FILE__ ) . 'includes/class-wishlist-dot-com-for-woocommerce-activator.php';
    Wishlist_Dot_Com_For_Woocommerce_Activator::activate();

    set_transient( 'wishlistdotcom_woo_get_started_notice_transient', true, 5 );
}

add_action( 'admin_notices', 'wishlistdotcom_woo_get_started_notice' );

/**
 * Admin notice after activation
 *
 * @return void
 */
function wishlistdotcom_woo_get_started_notice() {
    if( get_transient( 'wishlistdotcom_woo_get_started_notice_transient' ) ){

        $settings_url = admin_url( 'admin.php?page='.WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG );

        ?>
        <div class="notice notice-success is-dismissible">
            <p><?php echo  "Thank you for using \"WishList.com for WooCommerce\". <a href='".esc_url($settings_url)."'>Configure your WishList...</a>"; ?></p>
        </div>
        <?php
        delete_transient( 'wishlistdotcom_woo_get_started_notice_transient' );
    }
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wishlist-dot-com-for-woocommerce-deactivator.php
 */
function deactivate_Wishlist_Dot_Com_For_Woocommerce() {
    require_once plugin_dir_path( __FILE__ ) . 'includes/class-wishlist-dot-com-for-woocommerce-deactivator.php';
    Wishlist_Dot_Com_For_Woocommerce_Deactivator::deactivate();
}

/**
 * The code that runs during plugin uninstallation.
 * This action is documented in includes/class-wishlist-dot-com-for-woocommerce-uninstaller.php
 */
function uninstall_Wishlist_Dot_Com_For_Woocommerce() {
    require_once plugin_dir_path( __FILE__ ) . 'includes/class-wishlist-dot-com-for-woocommerce-uninstaller.php';
    Wishlist_Dot_Com_For_Woocommerce_Uninstaller::uninstall();
}
register_activation_hook( __FILE__, 'activate_Wishlist_Dot_Com_For_Woocommerce' );
register_deactivation_hook( __FILE__, 'deactivate_Wishlist_Dot_Com_For_Woocommerce' );
register_uninstall_hook( __FILE__, 'uninstall_Wishlist_Dot_Com_For_Woocommerce' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wishlist-dot-com-for-woocommerce.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 */
function run_Wishlist_Dot_Com_For_Woocommerce() {

    $plugin = new Wishlist_Dot_Com_For_Woocommerce();
    $plugin->run();

}
//Run after WooCommerce has been loaded. Other options include: plugins_loaded after_setup_theme woocommerce_init
add_action('woocommerce_init', 'run_Wishlist_Dot_Com_For_Woocommerce');

/* begin_plugin_updater */
/**
 * Plugin updater
 *
 */
require 'plugin-update-checker/plugin-update-checker.php';
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;
$Wishlist_Dot_Com_For_Woocommerce_myUpdateChecker = PucFactory::buildUpdateChecker(
    WISHLIST_DOT_COM_FOR_WOOCOMMERCE_UPDATE_URL,
    __FILE__, //Full path to the main plugin file or functions.php.
    WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG
);
//Add the license key to query arguments.
$Wishlist_Dot_Com_For_Woocommerce_myUpdateChecker->addQueryArgFilter('wishlistdotcom_filter_update_checks');
function wishlistdotcom_filter_update_checks($queryArgs) {
    //example default params installed_version=3.0.0&php=8.2.0&locale=en_US&checking_for_updates=1;
    $api_key = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_api_key');
    if ( !empty($api_key) ) {
        $queryArgs['api_key'] = $api_key;
    }
    $queryArgs['slug'] = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG;
    return $queryArgs;
}
/* end_plugin_updater */


/**
 * Gets information about the store for use in api calls to WishList.com
 *
 * @param $mode
 * @return array
 */
function wishlistdotcom_woo_store_id($params=[]){
    //this works since this plugin is loaded via 'run_Wishlist_Dot_Com_For_Woocommerce' above
    $store_url = wc_get_page_permalink('shop');
    $store_domain_a = wishlistdotcom_woo_url_to_domain_path($store_url);
    $store_url_no_scheme = $store_domain_a['url_no_scheme'];
    $store_domain = $store_domain_a['store_domain'];

    //set environment for api subdomain
    if(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_ENV == 'dev'){
        $env = 'd';
    }elseif(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_ENV == 'staging'){
        $env = 's';
    }else{
        $env = 'p';
    }

    if(isset($params['store_uuid'])){
        $store_uuid = $params['store_uuid'];
    }else{
        //store_uuid is created via wishlist-dot-com-for-woocommerce-admin.js from window message from wishlist.com
        $store_uuid = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_store_uuid');
    }
    if(isset($params['store_api_subdomain'])){
        $store_api_subdomain = $params['store_api_subdomain'];
    }else{
        //store_api_subdomain is created via wishlist-dot-com-for-woocommerce-admin.js from window message from wishlist.com
        $store_api_subdomain = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_store_api_subdomain');
    }
    if(isset($params['valid_api_key'])){
        $valid_api_key = $params['valid_api_key'];
    }else{
        $valid_api_key = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_valid_api_key');
    }
    if(isset($params['api_key'])){
        $api_key = $params['api_key'];
    }else{
        //api_key is created via wishlist-dot-com-for-woocommerce-admin.js from window message from wishlist.com
        $api_key = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_api_key');
    }

    //sub-domain codes
    //env: d = development, s = staging, p = production;
    //user interface mode: u = normal ui, p = popwin, i = iframe, n = iframe no ui
    //source: w = wishlist.com for woocommerce

    //dev.mystore.com/shop to dev-mystore-com-shop-apisuw.wishlist.com
    //$api_domain_codes = 'api[env][mode][source]-[storeid].wishlist.com';
    $api_domain_codes = '[store_api_subdomain]-[store_uuid]-a[env][mode][source].wishlist.com';

    $api_domain = str_replace(['[store_api_subdomain]','[env]','[mode]','[source]','[store_uuid]'],[$store_api_subdomain,$env,'u','w',$store_uuid],$api_domain_codes);
    //[mode] is left to be set later as necessary
    $api_domain_mode = str_replace(['[store_api_subdomain]','[env]','[source]','[store_uuid]'],[$store_api_subdomain,$env,'w',$store_uuid],$api_domain_codes);

    $return_a = [
        'env'=>$env,
        'api_domain'=>$api_domain,
        'api_domain_mode'=>$api_domain_mode,
        'store_uuid'=>$store_uuid,
        'store_url'=>$store_url,
        'store_url_no_scheme'=>$store_url_no_scheme,
        'store_domain'=>$store_domain,
        'api_key'=>$api_key,
        'valid_api_key'=>$valid_api_key,
        'store_api_subdomain'=>$store_api_subdomain,
    ];

    return $return_a;
}
/**
 * Register API enpoint
 *
 * @param $params
 * @return string[]
 */
function register_wishlistdotcom_woo_api() {
    register_rest_route('wishlistdotcom/v1', '/woo_api/', array(
        'methods' => WP_REST_Server::CREATABLE, //POST
        'callback' => 'wishlistdotcom_woo_api_endpoint',
        'permission_callback' => '__return_true' // For public access; change as per security requirement
    ));
}
add_action('rest_api_init', 'register_wishlistdotcom_woo_api');
/**
 * API enpoint to receive request from WishList.com to do an API call to WishList.com to process data and state changes
 *
 * @param $params
 * @return string[]
 */
function wishlistdotcom_woo_api_endpoint(WP_REST_Request $request){

    $api_action = sanitize_text_field($request->get_param('api_action'));

    if($api_action == 'get_wldcom_user_session'){
        $wldcom_user_session_id = sanitize_text_field($request['wldcom_user_session_id']);

        //Do api call to WishList.com and receive data. wldcom_user_session_id is valid for a temporary period and is WishList.com user specific
        $params['data'] = ['wldcom_user_session_id'=>$wldcom_user_session_id];
        $params['api_action'] = 'get_wldcom_user_session';
        //response is sanitized
        $response = wishlistdotcom_woo_api_call($params);

        if(isset($response['error'])){
            $wldcom_api_result = ['error'=>$response['error']];
        }else{
            $wldcom_api_result = $response;
        }

    }else{
        $wldcom_api_result = ['error'=>'no_matching_action'];
    }

    return new WP_REST_Response($wldcom_api_result, 200);
}
/**
 * Does an api call to WishList.com
 *
 * @param $params
 * @return mixed|string[]
 */
function wishlistdotcom_woo_api_call($params){
    $data = $params['data'];
    $api_action = $params['api_action'];

    $store_id_a = wishlistdotcom_woo_store_id();

    if($store_id_a['env'] == 'd'){
        $sslverify = false;
    }else{
        $sslverify = true;
    }
    if($api_action == 'check_service_level_wlcom_for_woocom'){
        //no need for secure login
        $api_url = 'https://'.$store_id_a['api_domain'].'/wooCommApi';
    }else{
        $api_url = 'https://'.$store_id_a['api_domain'].'/wooComSecureApiCall';
    }

    //send store's logo, url, button and other preferences to wishlist.com for display on wishlist.com
    $api_key = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_api_key');

    if(!empty($store_id_a['api_key']) && !empty($store_id_a['store_uuid']) ){
        $api_body = array(
            'store_uuid' => $store_id_a['store_uuid'],
            'credentials' => base64_encode($store_id_a['store_uuid'] . ':' . $store_id_a['api_key']),
            'data' => wp_json_encode($data),
            'api_action' => $api_action,
        );
        $args = array(
            'headers' => array(
                'Content-Type' => 'application/json'
            ),
            'body'=> wp_json_encode($api_body),
            'sslverify' => $sslverify,
        );
        $response = wp_remote_post($api_url,$args);

        if (is_wp_error($response)) {
            // Handle the error
            $error_message = $response->get_error_message();

            $return_a = [
                'error'=>$error_message,
            ];
        } else {
            // Request was successful
            $response_code = wp_remote_retrieve_response_code($response);
            $response_body = wp_remote_retrieve_body($response);

            // Parse the response body as needed (e.g., JSON or XML)
            $parsed_data = json_decode($response_body, true); // For JSON response

            if ($response_code === 200) {
                // Process the parsed data
                $return_a = $parsed_data;
            }else{
                $return_a = [
                    'error'=>$response_code,
                ];
            }
        }
    }else{
        $return_a = [
            'error'=>'no api_key',
        ];
    }

    //sanitize response from WishList.com
    if(!empty($return_a)){
        array_walk_recursive($return_a, function (&$item, $key) {
            if (is_string($item)) {
                $item = sanitize_text_field($item);
            }
        });
    }

    return $return_a;
}
